@@ -214,8 +214,20 @@ struct bbm1_cp
       double tol_piv;
       /* relative tolerance which is used to choose the pivot element
          of the simplex table */
-      int iter_max;
-      /* maximal allowed number of iterations (0 means no limit) */
+      int it_lim;
+      /* simplex iterations limit; if this value is positive, it is
+         decreased by one each time when one simplex iteration has been
+         performed, and reaching zero value signals the solver to stop
+         the search; negative value means no iterations limit */
+      int it_cnt;
+      /* simplex iterations count; this count is increased by one each
+         time when one simplex iteration has been performed */
+      double tm_lim;
+      /* searching time limit, in seconds; if this value is positive,
+         it is decreased each time when one simplex iteration has been
+         performed by the amount of time spent for the iteration, and
+         reaching zero value signals the solver to stop the search;
+         negative value means no time limit */
       int round;
       /* if this flag is set, the solver replaces computed values of
          basic continuous variables which are close to zero by exact
