@@ -36,6 +36,7 @@
 #define print                 _glp_print
 #define fault                 _glp_fault
 #define _insist               _glp_insist
+#define gmt2000               _glp_gmt2000
 #define watch                 _glp_watch
 
 #define umalloc               _glp_umalloc
@@ -74,8 +75,8 @@ struct ENV
       /* total number of currently allocated memory blocks */
       int mem_cpeak;
       /* peak value of mem_count */
-      double last_watch;
-      /* the value returned by the routine watch on the last call */
+      double start;
+      /* the time, in gmt2000() format, when the program started */
 };
 
 extern int init_lib_env(void);
@@ -99,6 +100,9 @@ extern void fault(char *fmt, ...);
 extern void _insist(char *expr, char *file, int line);
 /* check for logical condition */
 
+extern double gmt2000(void);
+/* determine the current time */
+
 extern double watch(void);
 /* take reading of stop-watch */
 
