@@ -40,6 +40,7 @@
 #define lpx_set_col_name      _glp_lpx_set_col_name
 #define lpx_set_row_bnds      _glp_lpx_set_row_bnds
 #define lpx_set_col_bnds      _glp_lpx_set_col_bnds
+#define lpx_set_class         _glp_lpx_set_class
 #define lpx_set_col_kind      _glp_lpx_set_col_kind
 #define lpx_set_obj_dir       _glp_lpx_set_obj_dir
 #define lpx_set_obj_c0        _glp_lpx_set_obj_c0
@@ -48,6 +49,7 @@
 #define lpx_set_row_stat      _glp_lpx_set_row_stat
 #define lpx_set_col_stat      _glp_lpx_set_col_stat
 #define lpx_load_mat          _glp_lpx_load_mat
+#define lpx_load_mat3         _glp_lpx_load_mat3
 #define lpx_set_mat_row       _glp_lpx_set_mat_row
 #define lpx_set_mat_col       _glp_lpx_set_mat_col
 #define lpx_unmark_all        _glp_lpx_unmark_all
@@ -59,12 +61,15 @@
 
 #define lpx_get_num_rows      _glp_lpx_get_num_rows
 #define lpx_get_num_cols      _glp_lpx_get_num_cols
+#define lpx_get_num_int       _glp_lpx_get_num_int
+#define lpx_get_num_bin       _glp_lpx_get_num_bin
 #define lpx_get_num_nz        _glp_lpx_get_num_nz
 #define lpx_get_prob_name     _glp_lpx_get_prob_name
 #define lpx_get_row_name      _glp_lpx_get_row_name
 #define lpx_get_col_name      _glp_lpx_get_col_name
 #define lpx_get_row_bnds      _glp_lpx_get_row_bnds
 #define lpx_get_col_bnds      _glp_lpx_get_col_bnds
+#define lpx_get_class         _glp_lpx_get_class
 #define lpx_get_col_kind      _glp_lpx_get_col_kind
 #define lpx_get_obj_dir       _glp_lpx_get_obj_dir
 #define lpx_get_obj_c0        _glp_lpx_get_obj_c0
@@ -78,7 +83,10 @@
 #define lpx_get_row_info      _glp_lpx_get_row_info
 #define lpx_get_col_info      _glp_lpx_get_col_info
 #define lpx_get_obj_val       _glp_lpx_get_obj_val
-#define lpx_print_sol         _glp_lpx_print_sol
+#define lpx_get_mip_stat      _glp_lpx_get_mip_stat
+#define lpx_get_mip_row       _glp_lpx_get_mip_row
+#define lpx_get_mip_col       _glp_lpx_get_mip_col
+#define lpx_get_mip_obj       _glp_lpx_get_mip_obj
 
 #define lpx_reset_parms       _glp_lpx_reset_parms
 #define lpx_set_int_parm      _glp_lpx_set_int_parm
@@ -97,18 +105,23 @@
 #define lpx_prim_art          _glp_lpx_prim_art
 #define lpx_dual_opt          _glp_lpx_dual_opt
 #define lpx_simplex           _glp_lpx_simplex
+#define lpx_integer           _glp_lpx_integer
 
 #define lpx_eval_tab_row      _glp_lpx_eval_tab_row
 #define lpx_eval_tab_col      _glp_lpx_eval_tab_col
 
+#define lpx_read_mps          _glp_lpx_read_mps
+#define lpx_print_sol         _glp_lpx_print_sol
+#define lpx_print_mip         _glp_lpx_print_mip
+
 /*----------------------------------------------------------------------
--- The structure LPX defines a low-level LP problem object and includes
--- the following information:
+-- The structure LPX defines LP/MIP problem object, which includes the
+-- following information:
 --
---    LP problem data;
+--    LP/MIP problem data;
 --    current basis;
 --    factorization of the basis matrix;
---    basic solution components;
+--    LP/MIP solution components;
 --    control parameters and statistics.
 --
 -- LP problem has the following formulation:
@@ -264,7 +277,7 @@ typedef struct LPX LPX;
    MAY DAMAGE ITS INTEGRITY AND CAUSE UNPREDICTABLE ERRORS! */
 
 struct LPX
-{     /* LP problem object */
+{     /* LP/MIP problem object */
       int m_max;
       /* maximal number of rows (if necessary, this parameter can be
          automatically changed) */
@@ -281,17 +294,21 @@ struct LPX
       /* working buffer used to return character strings */
       /*--------------------------------------------------------------*/
       /* LP problem data */
+      int class;
+      /* problem class: */
+#define LPX_LP          100   /* linear programming (LP) */
+#define LPX_MIP         101   /* mixed integer programming (MIP) */
       STR **name; /* STR *name[1+m+n]; */
       /* name[0] is a name of the LP problem object;
          name[k], 1 <= k <= m+n, is a name of the variable x[k] */
       int *typx; /* int typx[1+m_max+n_max]; */
       /* typx[0] is not used;
          typx[k], 1 <= k <= m+n, is the type of the variable x[k]: */
-#define LPX_FR          100   /* free variable:  -inf <  x[k] < +inf  */
-#define LPX_LO          101   /* lower bound:    l[k] <= x[k] < +inf  */
-#define LPX_UP          102   /* upper bound:    -inf <  x[k] <= u[k] */
-#define LPX_DB          103   /* double bound:   l[k] <= x[k] <= u[k] */
-#define LPX_FX          104   /* fixed variable: l[k]  = x[k]  = u[k] */
+#define LPX_FR          110   /* free variable:  -inf <  x[k] < +inf  */
+#define LPX_LO          111   /* lower bound:    l[k] <= x[k] < +inf  */
+#define LPX_UP          112   /* upper bound:    -inf <  x[k] <= u[k] */
+#define LPX_DB          113   /* double bound:   l[k] <= x[k] <= u[k] */
+#define LPX_FX          114   /* fixed variable: l[k]  = x[k]  = u[k] */
       double *lb; /* double lb[1+m_max+n_max]; */
       /* lb[0] is not used;
          lb[k], 1 <= k <= m+n, is an lower bound of the variable x[k];
@@ -314,8 +331,8 @@ struct LPX
          purposes */
       int dir;
       /* optimization direction (sense of the objective function): */
-#define LPX_MIN         110   /* minimization */
-#define LPX_MAX         111   /* maximization */
+#define LPX_MIN         120   /* minimization */
+#define LPX_MAX         121   /* maximization */
       double *coef; /* double coef[1+m_max+n_max]; */
       /* coef[0] is a constant term of the objective function;
          coef[k], 1 <= k <= m+n, is a coefficient of the objective
@@ -327,30 +344,30 @@ struct LPX
       /* current basis and basic solution components */
       int b_stat;
       /* status of the current basis: */
-#define LPX_B_UNDEF     120   /* the current basis is undefined */
-#define LPX_B_VALID     121   /* the current basis is valid */
+#define LPX_B_UNDEF     130   /* current basis is undefined */
+#define LPX_B_VALID     131   /* current basis is valid */
       int p_stat;
       /* status of the primal solution: */
-#define LPX_P_UNDEF     130   /* the status is undefined */
-#define LPX_P_FEAS      131   /* the solution is primal feasible */
-#define LPX_P_INFEAS    132   /* the solution is primal infeasible */
-#define LPX_P_NOFEAS    133   /* no primal feasible solution exists */
+#define LPX_P_UNDEF     132   /* primal status is undefined */
+#define LPX_P_FEAS      133   /* solution is primal feasible */
+#define LPX_P_INFEAS    134   /* solution is primal infeasible */
+#define LPX_P_NOFEAS    135   /* no primal feasible solution exists */
       int d_stat;
       /* status of the dual solution: */
-#define LPX_D_UNDEF     140   /* the status is undefined */
-#define LPX_D_FEAS      141   /* the solution is dual feasible */
-#define LPX_D_INFEAS    142   /* the solution is dual infeasible */
-#define LPX_D_NOFEAS    143   /* no dual feasible solution exists */
+#define LPX_D_UNDEF     136   /* dual status is undefined */
+#define LPX_D_FEAS      137   /* solution is dual feasible */
+#define LPX_D_INFEAS    138   /* solution is dual infeasible */
+#define LPX_D_NOFEAS    139   /* no dual feasible solution exists */
       int *tagx; /* int tagx[1+m_max+n_max]; */
       /* tagx[0] is not used;
          tagx[k], 1 <= k <= m+n, is the status of the variable x[k]
          (contents of this array is always defined independently on the
          status of the current basis): */
-#define LPX_BS          150   /* basic variable */
-#define LPX_NL          151   /* non-basic variable on lower bound */
-#define LPX_NU          152   /* non-basic variable on upper bound */
-#define LPX_NF          153   /* non-basic free variable */
-#define LPX_NS          154   /* non-basic fixed variable */
+#define LPX_BS          140   /* basic variable */
+#define LPX_NL          141   /* non-basic variable on lower bound */
+#define LPX_NU          142   /* non-basic variable on upper bound */
+#define LPX_NF          143   /* non-basic free variable */
+#define LPX_NS          144   /* non-basic fixed variable */
       int *posx; /* int posx[1+m_max+n_max]; */
       /* posx[0] is not used;
          posx[k], 1 <= k <= m+n, is the position of the variable x[k]
@@ -382,15 +399,22 @@ struct LPX
          xN[j] */
       /*--------------------------------------------------------------*/
       /* mixed integer programming (MIP) segment */
-      int mip;
-      /* if this flag is set, this problem is MIP problem; if this flag
-         is clear, this problem is pure LP problem */
       int *kind; /* int kind[1+n_max]; */
       /* kind[0] is not used;
          kind[j], 1 <= j <= n, specifies the kind of the j-th structural
          variable: */
-#define LPX_CV          160   /* continuous variable */
-#define LPX_IV          161   /* integer variable */
+#define LPX_CV          150   /* continuous variable */
+#define LPX_IV          151   /* integer variable */
+      int i_stat;
+      /* status of the integer solution: */
+#define LPX_I_UNDEF     160   /* integer status is undefined */
+#define LPX_I_OPT       161   /* solution is integer optimal */
+#define LPX_I_FEAS      162   /* solution is integer feasible */
+#define LPX_I_NOFEAS    163   /* no integer solution exists */
+      double *mipx; /* double mipx[1+m+n]; */
+      /* mipx[0] is not used;
+         mipx[k], 1 <= k <= m+n, is an (unscaled!) value of the variable
+         x[k] for integer optimal or integer feasible solution */
       /*--------------------------------------------------------------*/
       /* control parameters and statistics */
       int msg_lev;
@@ -443,9 +467,9 @@ struct LPX
       int round;
       /* solution rounding option:
          0 - report all computed values and reduced costs "as is"
-         1 - round computed values of basic variables and reduced costs
-             of non-basic variables toward their bounds or toward zero,
-             if possible (i.e. if it is allowed by the tolerances) */
+         1 - if possible (allowed by the tolerances), replace computed
+             values and reduced costs which are close to zero by exact
+             zeros */
       double obj_ll;
       /* lower limit of the objective function; if on the phase II the
          objective function reaches this limit and continues decreasing,
@@ -476,6 +500,23 @@ struct LPX
       /* output delay, in seconds; this parameter specifies how long
          the solver should delay sending information about the solution
          to the standard output; zero value means no delay */
+      int branch; /* MIP */
+      /* branching heuristic:
+         0 - branch on the first variable
+         1 - branch on the last variable
+         2 - branch using a heuristic by Driebeck and Tomlin */
+      int btrack; /* MIP */
+      /* backtracking heuristic:
+         0 - depth first search
+         1 - breadth first search
+         2 - backtrack using the best projection heuristic */
+      double tol_int; /* MIP */
+      /* absolute tolerance used to check if the current basic solution
+         is integer feasible */
+      double tol_obj; /* MIP */
+      /* relative tolerance used to check if the value of the objective
+         function is not better than in the best known integer feasible
+         solution */
 };
 
 /* status codes reported by the routine lpx_get_status: */
@@ -517,6 +558,10 @@ struct LPX
 #define LPX_K_TMLIM     213   /* lp->tm_lim */
 #define LPX_K_OUTFRQ    214   /* lp->out_frq */
 #define LPX_K_OUTDLY    215   /* lp->out_dly */
+#define LPX_K_BRANCH    216   /* lp->branch */
+#define LPX_K_BTRACK    217   /* lp->btrack */
+#define LPX_K_TOLINT    218   /* lp->tol_int */
+#define LPX_K_TOLOBJ    219   /* lp->tol_obj */
 
 /* problem creating and modifying routines ---------------------------*/
 
@@ -550,6 +595,9 @@ void lpx_set_row_bnds(LPX *lp, int i, int typx, double lb, double ub);
 void lpx_set_col_bnds(LPX *lp, int j, int typx, double lb, double ub);
 /* set (change) column bounds */
 
+void lpx_set_class(LPX *lp, int class);
+/* set (change) problem class */
+
 void lpx_set_col_kind(LPX *lp, int j, int kind);
 /* set (change) column kind */
 
@@ -575,6 +623,9 @@ void lpx_load_mat(LPX *lp,
       void *info, double (*mat)(void *info, int *i, int *j));
 /* load constraint matrix */
 
+void lpx_load_mat3(LPX *lp, int nz, int rn[], int cn[], double a[]);
+/* load constraint matrix */
+
 void lpx_set_mat_row(LPX *lp, int i, int len, int ndx[], double val[]);
 /* set (replace) row of the constraint matrix */
 
@@ -607,6 +658,12 @@ int lpx_get_num_rows(LPX *lp);
 int lpx_get_num_cols(LPX *lp);
 /* determine number of columns */
 
+int lpx_get_num_int(LPX *lp);
+/* determine number of integer columns */
+
+int lpx_get_num_bin(LPX *lp);
+/* determine number of binary columns */
+
 int lpx_get_num_nz(LPX *lp);
 /* determine number of constraint coefficients */
 
@@ -627,6 +684,9 @@ void lpx_get_col_bnds(LPX *lp, int j, int *typx, double *lb,
       double *ub);
 /* obtain column bounds */
 
+int lpx_get_class(LPX *lp);
+/* query problem class */
+
 int lpx_get_col_kind(LPX *lp, int j);
 /* query column kind */
 
@@ -657,19 +717,28 @@ int lpx_get_prim_stat(LPX *lp);
 int lpx_get_dual_stat(LPX *lp);
 /* query dual status of basic solution */
 
-void lpx_get_row_info(LPX *lp, int i, int *tagx, double *valx,
-      double *dx);
+void lpx_get_row_info(LPX *lp, int i, int *tagx, double *vx, double *dx)
+      ;
 /* obtain row solution information */
 
-void lpx_get_col_info(LPX *lp, int j, int *tagx, double *valx,
-      double *dx);
+void lpx_get_col_info(LPX *lp, int j, int *tagx, double *vx, double *dx)
+      ;
 /* obtain column solution information */
 
 double lpx_get_obj_val(LPX *lp);
 /* obtain value of the objective function */
 
-int lpx_print_sol(LPX *lp, char *fname);
-/* write problem solution in printable format */
+int lpx_get_mip_stat(LPX *lp);
+/* query status of MIP solution */
+
+double lpx_get_mip_row(LPX *lp, int i);
+/* obtain row activity for MIP solution */
+
+double lpx_get_mip_col(LPX *lp, int j);
+/* obtain column activity for MIP solution */
+
+double lpx_get_mip_obj(LPX *lp);
+/* obtain value of the obj. func. for MIP solution */
 
 /* control parameters and statistics routines ------------------------*/
 
@@ -721,6 +790,9 @@ int lpx_dual_opt(LPX *lp);
 int lpx_simplex(LPX *lp);
 /* easy-to-use driver to the simplex method */
 
+int lpx_integer(LPX *lp);
+/* easy-to-use driver to the branch-and-bound method */
+
 /* simplex table routines --------------------------------------------*/
 
 int lpx_eval_tab_row(LPX *lp, int k, int ndx[], double val[]);
@@ -729,6 +801,17 @@ int lpx_eval_tab_row(LPX *lp, int k, int ndx[], double val[]);
 int lpx_eval_tab_col(LPX *lp, int k, int ndx[], double val[]);
 /* compute column of the simplex table */
 
+/* additional utility routines ---------------------------------------*/
+
+LPX *lpx_read_mps(char *fname);
+/* read in problem data using MPS format */
+
+int lpx_print_sol(LPX *lp, char *fname);
+/* write LP problem solution in printable format */
+
+int lpx_print_mip(LPX *lp, char *fname);
+/* write MIP problem solution in printable format */
+
 #endif
 
 /* eof */
