@@ -63,7 +63,6 @@
 #define rsm_dual              _glp_rsm_dual
 #define rsm_feas              _glp_rsm_feas
 #define rsm_primal            _glp_rsm_primal
-#define rsm1_driver           _glp_rsm1_driver
 #define scale_rsm             _glp_scale_rsm
 #define update_b              _glp_update_b
 #define update_dvec           _glp_update_dvec
@@ -251,50 +250,6 @@ extern int rsm_feas(RSM *rsm, int (*monit)(void),
       int relax);
 /* find feasible solution using primal simplex method */
 
-struct rsm1_cp
-{     /* control parameters passed to the rsm1_driver routine */
-      int what;
-      /* this parameter specifies what basis solution should be found
-         by the solver:
-         0 - initial solution
-         1 - primal feasible solution
-         2 - optimal solution */
-      int scale;
-      /* if this flag is set, the solver performs automatic scaling the
-         problem; otherwise the scaling is not used */
-      int dual;
-      /* if this flag is set, the solver uses the dual simplex method
-         in order to find the primal feasible basis solution; otherwise
-         the solver uses the primal simplex method */
-      int steep;
-      /* if this flag is set, the solver uses the steepest edge pricing
-         (for both primal and dual simplex) proposed by Goldfarb & Reid;
-         otherwise the solver uses the standard "textbook" pricing */
-      int relax;
-      /* if this flag is set, the solver uses two-pass ratio test (for
-         both primal and dual simplex) proposed by P.Harris; otherwise
-         the solver uses the standard "textbook" ratio test */
-      double tol_bnd;
-      /* relative tolerance which is used to see if the solution is
-         primal feasible */
-      double tol_dj;
-      /* relative tolerance which is used to see if the solution is
-         dual feasible */
-      double tol_piv;
-      /* relative tolerance which is used to choose the pivot element
-         of the simplex table */
-      int iter_max;
-      /* maximal allowed number of iterations (0 means no limit) */
-      int round;
-      /* if this flag is set, the solver replaces computed values of
-         basic variables and reduced costs of non-basic variables which
-         are close to zero by exact zeros; otherwise the solver remains
-         all computed values and reduced costs "as is" */
-};
-
-extern int rsm1_driver(LP *lp, LPSOL *sol, struct rsm1_cp *cp);
-/* driver for the revised simplex method */
-
 /*** auxiliary routines ***/
 
 extern RSM *create_rsm(LPI *lp);
