@@ -34,8 +34,6 @@
 #include "glplang.h"
 #include "glpmps.h"
 
-#define error print
-
 /*----------------------------------------------------------------------
 -- glp_check_data - turn on/off data consistency checking flag.
 --
@@ -1206,7 +1204,7 @@ int glp_print_soln(LPI *lp, char *fname)
          fname);
       fp = fopen(fname, "w");
       if (fp == NULL)
-      {  error("glp_print_soln: can't create `%s' - %s", fname,
+      {  print("glp_print_soln: can't create `%s' - %s", fname,
             strerror(errno));
          goto fail;
       }
@@ -1351,7 +1349,7 @@ int glp_print_soln(LPI *lp, char *fname)
       fprintf(fp, "End of output\n");
       fflush(fp);
       if (ferror(fp))
-      {  error("glp_print_soln: can't write to `%s' - %s", fname,
+      {  print("glp_print_soln: can't write to `%s' - %s", fname,
             strerror(errno));
          goto fail;
       }
@@ -1528,7 +1526,7 @@ LPI *glp_read_lpm1(char *infile, char *outfile)
          for (e = form; e != NULL; e = e->next)
          {  if (e->j == 0)
             {  if (cons->type == 'F')
-               {  error("glp_read_lpm1: free row `%s' has constant term"
+               {  print("glp_read_lpm1: free row `%s' has constant term"
                      , name);
                   goto fail;
                }
@@ -1552,7 +1550,7 @@ LPI *glp_read_lpm1(char *infile, char *outfile)
       /* model has been read successfully */
       return lp;
 fail: /* the operation failed */
-      error("glp_read_lpm1: processing terminated due to errors");
+      print("glp_read_lpm1: processing terminated due to errors");
       if (lp != NULL) glp_delete_prob(lp);
       if (prob != NULL) delete_prob(prob);
       if (pdb != NULL) terminate();
@@ -1606,14 +1604,14 @@ LPI *glp_read_mps1(char *fname)
          else if (strcmp(row->type, "E") == 0)
             type = 'S';
          else
-         {  error("glp_read_mps1: row `%s' has unknown type `%s'",
+         {  print("glp_read_mps1: row `%s' has unknown type `%s'",
                row->name, row->type);
             goto fail;
          }
          glp_set_row_bnds(lp, i, type, 0.0, 0.0);
       }
       if (obj == 0)
-      {  error("glp_read_mps1: objective row not found");
+      {  print("glp_read_mps1: objective row not found");
          goto fail;
       }
       /* process COLUMN section */
@@ -1670,7 +1668,7 @@ LPI *glp_read_mps1(char *fname)
             glp_get_row_bnds(lp, cqe->ind, &type, &lb, &ub);
             switch (type)
             {  case 'F':
-                  error("glp_read_mps1: range vector entry refers to ro"
+                  print("glp_read_mps1: range vector entry refers to ro"
                      "w `%s' of N type", mps->row[cqe->ind]->name);
                   goto fail;
                case 'L':
@@ -1724,7 +1722,7 @@ LPI *glp_read_mps1(char *fname)
                lb = 0.0, ub = 1.0;
             }
             else
-            {  error("glp_read_mps1: bound vector entry for column `%s'"
+            {  print("glp_read_mps1: bound vector entry for column `%s'"
                   " has unknown type `%s'",
                   mps->col[bqe->ind]->name, bqe->type);
                goto fail;
@@ -2314,7 +2312,8 @@ LPX *glp_lpi_to_lpx(LPI *lpi, int all)
       /* copy column kinds */
       for (j = 1; j <= n; j++)
       {  if (glp_get_col_kind(lpi, j) == 'I')
-         {  for (j = 1; j <= n; j++)
+         {  lpx_set_class(lpx, LPX_MIP);
+            for (j = 1; j <= n; j++)
             {  switch (glp_get_col_kind(lpi, j))
                {  case 'C': kind = LPX_CV; break;
                   case 'I': kind = LPX_IV; break;
@@ -2528,32 +2527,32 @@ int glp_write_mps(LPI *lp, char *fname, struct wmps *parm)
       }
       /* check control parameters for correctness */
       if (!(parm->prob_info == 0 || parm->prob_info == 1))
-      {  error("glp_write_mps: prob_info = %d; invalid parameter",
+      {  print("glp_write_mps: prob_info = %d; invalid parameter",
             parm->prob_info);
          return 1;
       }
       if (!(0 <= parm->make_obj && parm->make_obj <= 2))
-      {  error("glp_write_mps: make_obj = %d; invalid parameter",
+      {  print("glp_write_mps: make_obj = %d; invalid parameter",
             parm->make_obj);
          return 1;
       }
       if (!(parm->use_names == 0 || parm->use_names == 1))
-      {  error("glp_write_mps: use_names = %d; invalid parameter",
+      {  print("glp_write_mps: use_names = %d; invalid parameter",
             parm->use_names);
          return 1;
       }
       if (!(parm->one_entry == 0 || parm->one_entry == 1))
-      {  error("glp_write_mps: one_entry = %d; invalid parameter",
+      {  print("glp_write_mps: one_entry = %d; invalid parameter",
             parm->one_entry);
          return 1;
       }
       if (!(parm->pedantic == 0 || parm->pedantic == 1))
-      {  error("glp_write_mps: pedantic = %d; invalid parameter",
+      {  print("glp_write_mps: pedantic = %d; invalid parameter",
             parm->pedantic);
          return 1;
       }
       if (!(parm->skip_empty == 0 || parm->skip_empty == 1))
-      {  error("glp_write_mps: prob_info = %d; invalid parameter",
+      {  print("glp_write_mps: prob_info = %d; invalid parameter",
             parm->skip_empty);
          return 1;
       }
@@ -2562,7 +2561,7 @@ int glp_write_mps(LPI *lp, char *fname, struct wmps *parm)
       /* open the output text file */
       fp = fopen(fname, "w");
       if (fp == NULL)
-      {  error("glp_write_mps: can't create `%s' - %s", fname,
+      {  print("glp_write_mps: can't create `%s' - %s", fname,
             strerror(errno));
          goto fail;
       }
@@ -2571,11 +2570,11 @@ int glp_write_mps(LPI *lp, char *fname, struct wmps *parm)
       ncols = glp_get_num_cols(lp);
       /* the problem should contain at least one row and one column */
       if (nrows == 0)
-      {  error("glp_write_mps: problem has no rows");
+      {  print("glp_write_mps: problem has no rows");
          goto fail;
       }
       if (ncols == 0)
-      {  error("glp_write_mps: problem has no columns");
+      {  print("glp_write_mps: problem has no columns");
          goto fail;
       }
       /* determine whether the routine should output objective function
@@ -2813,7 +2812,7 @@ int glp_write_mps(LPI *lp, char *fname, struct wmps *parm)
       /* close the output text file */
       fflush(fp);
       if (ferror(fp))
-      {  error("glp_write_mps: can't write to `%s' - %s", fname,
+      {  print("glp_write_mps: can't write to `%s' - %s", fname,
             strerror(errno));
          goto fail;
       }
