@@ -25,11 +25,9 @@
 
 #include <float.h>
 #include <math.h>
-#include <time.h>
+#include <stddef.h>
 #include "glpbbm.h"
 
-#define error print
-
 /*----------------------------------------------------------------------
 -- bbm1_driver - driver for the branch-and-bound method.
 --
@@ -79,9 +77,11 @@
 --
 -- 0 - no errors. This case means that the solver has successfully
 --     finished solving the problem;
--- 1 - iteration limit exceeded. In this case the solver reports the
---     recently obtained basis solution;
--- 2 - numerical unstability or problems with the basis matrix. This
+-- 1 - iterations limit exceeded. In this case the solver reports the
+--     best known integer feasible solution;
+-- 2 - time limit exceeded. In this case the solver reports the best
+--     known integer feasible solution;
+-- 3 - numerical unstability or problems with the basis matrix. This
 --     case means that the solver is not able to solve the problem. */
 
 static struct bbm1_cp *cp;
@@ -111,7 +111,10 @@ static double *gvec; /* double gvec[1+n]; */
 static int *work; /* int work[1+m+n]; */
 /* auxiliary working array */
 
-static clock_t t_last;
+static double start;
+/* the search starting time */
+
+static double t_last;
 /* most recent time at which visual information was displayed */
 
 /*----------------------------------------------------------------------
@@ -198,7 +201,7 @@ static void initialize(LP *mip, LPSOL *sol)
       bb->rsm->indn = ucalloc(1+n, sizeof(int));
       bb->rsm->tagn = ucalloc(1+n, sizeof(int));
       bb->rsm->inv = inv_create(m, 100);
-      bb->rsm->iter = 0;
+      bb->rsm->iter = cp->it_cnt;
       /* so far as the RSM arrays type, lb, ub, posx, indb, indn, and
          tagn will be set later for each individual subproblem, these
          arrays are not initialized here */
@@ -242,8 +245,9 @@ static void initialize(LP *mip, LPSOL *sol)
          dvec = gvec = NULL;
       /* allocate auxiliary working array */
       work = ucalloc(1+m+n, sizeof(int));
-      /* reset timer */
-      t_last = 0;
+      /* reset timers */
+      start = watch();
+      t_last = -1.0;
       /* create the root problem */
       node = get_atom(bb->node_pool);
       node->up = NULL;
@@ -307,7 +311,7 @@ static void display(void)
          print("+%6d: mip = %17.9e; lp = %17.9e (%d; %d)",
             bb->rsm->iter, s * bb->best, s * bb->root->objval,
             bb->count, bb->total);
-      t_last = clock();
+      t_last = watch();
       return;
 }
 
@@ -349,12 +353,16 @@ static double eval_objfun(void)
 static int why;
 /* this flag explains why the monitoring routine returned non-zero:
    0 - further serach is useless
-   1 - iteration limit exhausted */
+   1 - iterations limit exhausted
+   2 - time limit exhausted */
 
 static int dual_monit(void)
 {     int ret = 0;
-      /* display visual information (once per second) */
-      if (t_last == 0 || clock() - t_last > CLOCKS_PER_SEC) display();
+      double spent;
+      /* determine the spent amount of time */
+      spent = watch() - start;
+      /* display visual information (once per three seconds) */
+      if (t_last < 0.0 || watch() - t_last >= 2.9) display();
       /* if the current value of the objective function is not better
          than for the best integer feasible solution, further search is
          useless and may be terminated */
@@ -365,12 +373,16 @@ static int dual_monit(void)
             ret = 1;
          }
       }
-      /* if total number of simplex iterations exceeds allowed maximum,
-         the search should be terminated */
-      if (cp->iter_max && bb->rsm->iter > cp->iter_max)
+      /* check if the iterations limit has been exhausted */
+      if (cp->it_lim >= 0 && bb->rsm->iter >= cp->it_lim)
       {  why = 1;
          ret = 1;
       }
+      /* check if the time limit has been exhausted */
+      if (cp->tm_lim >= 0 && spent >= cp->tm_lim)
+      {  why = 2;
+         ret = 1;
+      }
       return ret;
 }
 
@@ -389,19 +401,27 @@ static int dual_monit(void)
 -- 1 - either problem has no (primal) feasible solution or its optimal
 --     solution is not better that the best integer feasible solution
 --     which is currently known;
--- 2 - numerical stability lost (the current basis solution became dual
+-- 2 - numerical instability (the current basis solution became dual
 --     infeasible due to round-off errors);
 -- 3 - numerical problems with basis matrix (the current basis matrix
 --     became singular or ill-conditioned due to unsuccessful choice of
 --     the pivot element on the last simplex iteration);
--- 4 - number of simplex iterations exceeds allowable maximum. */
+-- 4 - iterations limit exceeded;
+-- 5 - time limit exceeded. */
 
 static int dual_simplex(void)
 {     int ret;
       /* call the dual simplex method driver routine */
       ret = rsm_dual(bb->rsm, dual_monit, c, cp->tol_bnd, cp->tol_dj,
          cp->tol_piv, dvec, cp->relax);
-      if (ret == 4 && why == 0) ret = 1;
+      if (ret == 4)
+      {  switch (why)
+         {  case 0: ret = 1; break;
+            case 1: ret = 4; break;
+            case 2: ret = 5; break;
+            default: insist(why != why);
+         }
+      }
       /* return to the calling program */
       return ret;
 }
@@ -429,8 +449,11 @@ static int phase;
 
 static int primal_monit(void)
 {     int ret = 0;
-      /* display visual information (once per second) */
-      if (t_last == 0 || clock() - t_last > CLOCKS_PER_SEC)
+      double spent;
+      /* determine the spent amount of time */
+      spent = watch() - start;
+      /* display visual information (once per three seconds) */
+      if (t_last < 0.0 || watch() - t_last >= 2.9)
       {  int i, j, k, defect;
          double objval, infsum;
          /* compute current values of basic variables */
@@ -467,11 +490,18 @@ static int primal_monit(void)
          print("%c%6d:   objval = %17.9e   infsum = %17.9e (%d)",
             phase == 1 ? ' ' : '*', bb->rsm->iter, objval, infsum,
             defect);
-         t_last = clock();
+         t_last = watch();
+      }
+      /* check if the iterations limit has been exhausted */
+      if (cp->it_lim >= 0 && bb->rsm->iter >= cp->it_lim)
+      {  why = 1;
+         ret = 1;
+      }
+      /* check if the time limit has been exhausted */
+      if (cp->tm_lim >= 0 && spent >= cp->tm_lim)
+      {  why = 2;
+         ret = 1;
       }
-      /* if total number of simplex iterations exceeds allowed maximum,
-         tell the primal simplex driver to terminate the search */
-      if (cp->iter_max && bb->rsm->iter > cp->iter_max) ret = 1;
       return ret;
 }
 
@@ -491,12 +521,13 @@ static int primal_monit(void)
 --
 -- 0 - optimal solution found;
 -- 1 - problem has no (primal) feasible solution;
--- 2 - numerical stability lost (the current basis solution became
---     primal infeasible due to round-off errors);
+-- 2 - numerical instability (the current basis solution became primal
+--     infeasible due to round-off errors);
 -- 3 - numerical problems with basis matrix (the current basis matrix
 --     became singular or ill-conditioned due to unsuccessful choice of
 --     the pivot element on the last simplex iteration);
--- 4 - number of simplex iterations exceeds allowable maximum. */
+-- 4 - iterations limit exceeded;
+-- 5 - time limit exceeded. */
 
 static int primal_simplex(void)
 {     int ret;
@@ -511,7 +542,15 @@ static int primal_simplex(void)
       ret = rsm_primal(bb->rsm, primal_monit, c, cp->tol_bnd,
          0.70 * cp->tol_dj, cp->tol_piv, gvec, cp->relax);
       insist(ret != 1);
-done: /* return to the calling program */
+done: /* analyze return code */
+      if (ret == 4)
+      {  switch (why)
+         {  case 1: ret = 4; break;
+            case 2: ret = 5; break;
+            default: insist(why != why);
+         }
+      }
+      /* return to the calling program */
       return ret;
 }
 
@@ -809,7 +848,8 @@ static void exempt_node(BBNODE *node)
 -- 3 - numerical problems with basis matrix (the current basis matrix
 --     became singular or ill-conditioned due to unsuccessful choice of
 --     the pivot element on the last simplex iteration);
--- 4 - number of simplex iterations exceeds allowable maximum. */
+-- 4 - iterations limit exceeded;
+-- 5 - time limit exceeded. */
 
 static int solve_problem(BBNODE *node, int warm)
 {     int ret, i, j;
@@ -846,8 +886,8 @@ static int solve_problem(BBNODE *node, int warm)
          start from any valid basis can be used in order to solve the
          problem */
       if (ret == 2)
-      {  print("Numerical stability lost; recovering...");
-         t_last = clock();
+      {  print("Numerical instability; recovering...");
+         t_last = watch();
          /* although the current (dual infeasible) basis may be used as
             initial basis, the practice shows that it's better to start
             again from the optimal basis of the parent problem */
@@ -872,13 +912,13 @@ static int solve_problem(BBNODE *node, int warm)
          fails; however in this case the primal simplex can be invoked
          once again */
       while (ret == 2)
-      {  print("Numerical stability lost (again); recovering...");
-         t_last = clock();
+      {  print("Numerical instability (again); recovering...");
+         t_last = watch();
          /* reinvert the current basis matrix (this usually helps to
             improve accuracy of the representation) */
          if (invert_b(bb->rsm))
-amba:    {  error("Numerical problems with basis matrix");
-            error("Sorry, basis recovery procedure not implemented");
+amba:    {  print("Numerical problems with basis matrix");
+            print("Sorry, basis recovery procedure not implemented");
             ret = 3;
             goto done;
          }
@@ -913,9 +953,14 @@ amba:    {  error("Numerical problems with basis matrix");
                dual simplex iteration */
             goto amba;
          case 4:
-            /* iteration limit exceeded; search terminated */
+            /* iterations limit exceeded; search terminated */
             display();
-            print("ITERATION LIMIT EXCEEDED; SEARCH TERMINATED");
+            print("ITERATIONS LIMIT EXCEEDED; SEARCH TERMINATED");
+            break;
+         case 5:
+            /* time limit exceeded; search terminated */
+            display();
+            print("TIME LIMIT EXCEEDED; SEARCH TERMINATED");
             break;
          default:
             insist(ret != ret);
@@ -1359,12 +1404,16 @@ loop: /* main loop starts here */
             /* the basis matrix became singular or ill-conditioned due
                to unsuccessful choice of the pivot element on the last
                dual simplex iteration */
-            ret = 2;
+            ret = 3;
             goto done;
          case 4:
-            /* iteration limit exceeded; search terminated */
+            /* iterations limit exceeded; search terminated */
             ret = 1;
             goto done;
+         case 5:
+            /* time limit exceeded; search terminated */
+            ret = 2;
+            goto done;
          default:
             insist(ret != ret);
       }
@@ -1489,7 +1538,17 @@ cut:  /* the current problem has been fathomed and should be removed
       warm = 0;
       /* continue the search */
       goto loop;
-done: /* terminate branch-and-bound environment */
+done: /* reflect the spent resources in the parameter block */
+      if (cp->it_lim >= 0)
+      {  cp->it_lim -= (bb->rsm->iter - cp->it_cnt);
+         if (cp->it_lim < 0) cp->it_lim = 0;
+      }
+      cp->it_cnt = bb->rsm->iter;
+      if (cp->tm_lim >= 0.0)
+      {  cp->tm_lim -= (watch() - start);
+         if (cp->tm_lim < 0.0) cp->tm_lim = 0.0;
+      }
+      /* terminate branch-and-bound environment */
       terminate();
       /* return to the calling program */
       return ret;
