@@ -392,7 +392,7 @@ int chol_numb(MAT *A, MAT *U, void *_head[], double _work[])
 -- Positive Definite Systems. Prentice-Hall, 1981.
 --
 -- See the acknowledgements in comments to the routines genqmd, qmdrch,
--- qmdqt, qmdupd, and qmdmrg included into the partition GLPQMD. */
+-- qmdqt, qmdupd, and qmdmrg in the module GLPQMD. */
 
 void min_deg(MAT *A, PER *P)
 {     ELEM *e;
