@@ -31,8 +31,6 @@
 #include "glphbsm.h"
 #include "glpstr.h"
 
-#define error print
-
 /*----------------------------------------------------------------------
 -- load_hbsm - load sparse matrix in Harwell-Boeing format.
 --
@@ -93,7 +91,7 @@ HBSM *load_hbsm(char *_fname)
       print("load_hbsm: reading matrix from `%s'...", fname);
       fp = fopen(fname, "r");
       if (fp == NULL)
-      {  error("load_hbsm: unable to open `%s' - %s", fname,
+      {  print("load_hbsm: unable to open `%s' - %s", fname,
             strerror(errno));
          goto fail;
       }
@@ -124,7 +122,7 @@ HBSM *load_hbsm(char *_fname)
       if (strchr("RCP",   hbsm->mxtype[0]) == NULL ||
           strchr("SUHZR", hbsm->mxtype[1]) == NULL ||
           strchr("AE",    hbsm->mxtype[2]) == NULL)
-      {  error("%s:%d: matrix type `%s' not recognised", fname, seqn,
+      {  print("%s:%d: matrix type `%s' not recognised", fname, seqn,
             hbsm->mxtype);
          goto fail;
       }
@@ -213,7 +211,7 @@ HBSM *load_hbsm(char *_fname)
             hbsm->rhsval)) goto fail;
       }
       else
-      {  error("%s:%d: right-hand side type `%c' not recognised",
+      {  print("%s:%d: right-hand side type `%c' not recognised",
             fname, seqn, hbsm->rhstyp[0]);
          goto fail;
       }
@@ -265,26 +263,26 @@ static int read_card(void)
       for (;;)
       {  c = fgetc(fp);
          if (ferror(fp))
-         {  error("%s:%d: read error - %s", fname, seqn,
+         {  print("%s:%d: read error - %s", fname, seqn,
                strerror(errno));
             return 1;
          }
          if (feof(fp))
          {  if (k == 0)
-               error("%s:%d: unexpected eof", fname, seqn);
+               print("%s:%d: unexpected eof", fname, seqn);
             else
-               error("%s:%d: missing final LF", fname, seqn);
+               print("%s:%d: missing final LF", fname, seqn);
             return 1;
          }
          if (c == '\r') continue;
          if (c == '\n') break;
          if (iscntrl(c))
-         {  error("%s:%d: invalid control character 0x%02X", fname,
+         {  print("%s:%d: invalid control character 0x%02X", fname,
                seqn, c);
             return 1;
          }
          if (k == 80)
-         {  error("%s:%d: card image too long", fname, seqn);
+         {  print("%s:%d: card image too long", fname, seqn);
             return 1;
          }
          card[k++] = (char)c;
@@ -306,7 +304,7 @@ static int scan_int(char *fld, int pos, int width, int *val)
       insist(1 <= width && width <= 80);
       memcpy(str, card+pos, width), str[width] = '\0';
       if (str2int(strspx(str), val))
-      {  error("%s:%d: field `%s' contains invalid value `%s'",
+      {  print("%s:%d: field `%s' contains invalid value `%s'",
             fname, seqn, fld, str);
          return 1;
       }
@@ -333,7 +331,7 @@ static int parse_fmt(char *fmt)
       char str[80+1];
       /* first character should be left parenthesis */
       if (fmt[0] != '(')
-fail: {  error("load_hbsm: format `%s' not recognised", fmt);
+fail: {  print("load_hbsm: format `%s' not recognised", fmt);
          return 1;
       }
       k = 1;
@@ -413,7 +411,7 @@ static int read_int_array(char *name, char *fmt, int n, int val[])
       char str[80+1];
       if (parse_fmt(fmt)) return 1;
       if (!(fmt_f == 'I' && fmt_w <= 80 && fmt_k * fmt_w <= 80))
-      {  error("%s:%d: can't read array `%s' - invalid format `%s'",
+      {  print("%s:%d: can't read array `%s' - invalid format `%s'",
             fname, seqn, name, fmt);
          return 1;
       }
@@ -425,7 +423,7 @@ static int read_int_array(char *name, char *fmt, int n, int val[])
          memcpy(str, card + fmt_w * pos, fmt_w), str[fmt_w] = '\0';
          strspx(str);
          if (str2int(str, &val[k]))
-         {  error("%s:%d: can't read array `%s' - invalid value `%s'",
+         {  print("%s:%d: can't read array `%s' - invalid value `%s'",
                fname, seqn, name, str);
             return 1;
          }
@@ -447,7 +445,7 @@ static int read_real_array(char *name, char *fmt, int n, double val[])
       char str[80+1], *ptr;
       if (parse_fmt(fmt)) return 1;
       if (!(fmt_f != 'I' && fmt_w <= 80 && fmt_k * fmt_w <= 80))
-      {  error("%s:%d: can't read array `%s' - invalid format `%s'",
+      {  print("%s:%d: can't read array `%s' - invalid format `%s'",
             fname, seqn, name, fmt);
          return 1;
       }
@@ -459,7 +457,7 @@ static int read_real_array(char *name, char *fmt, int n, double val[])
          memcpy(str, card + fmt_w * pos, fmt_w), str[fmt_w] = '\0';
          strspx(str);
          if (strchr(str, '.') == NULL && strcmp(str, "0"))
-         {  error("%s(%d): can't read array `%s' - value `%s' has no de"
+         {  print("%s(%d): can't read array `%s' - value `%s' has no de"
                "cimal point", fname, seqn, name, str);
             return 1;
          }
@@ -478,7 +476,7 @@ static int read_real_array(char *name, char *fmt, int n, double val[])
             *ptr = 'E';
          }
          if (str2dbl(str, &val[k]))
-         {  error("%s:%d: can't read array `%s' - invalid value `%s'",
+         {  print("%s:%d: can't read array `%s' - invalid value `%s'",
                fname, seqn, name, str);
             return 1;
          }
@@ -545,46 +543,46 @@ MAT *hbsm_to_mat(HBSM *hbsm)
       int i, j, k;
       double val;
       if (hbsm->nrow < 1)
-      {  error("hbsm_to_mat: invalid number of rows");
+      {  print("hbsm_to_mat: invalid number of rows");
          goto fail;
       }
       if (hbsm->ncol < 1)
-      {  error("hbsm_to_mat: invalid number of columns");
+      {  print("hbsm_to_mat: invalid number of columns");
          goto fail;
       }
       if (strcmp(hbsm->mxtype, "RSA") == 0)
       {  /* real symmetric assembled */
          if (hbsm->nrow != hbsm->ncol)
-err1:    {  error("hbsm_to_mat: number of rows not equal to number of c"
+err1:    {  print("hbsm_to_mat: number of rows not equal to number of c"
                "olumns");
             goto fail;
          }
          if (hbsm->colptr == NULL)
-err2:    {  error("hbsm_to_mat: array `colptr' not allocated");
+err2:    {  print("hbsm_to_mat: array `colptr' not allocated");
             goto fail;
          }
          if (hbsm->rowind == NULL)
-err3:    {  error("hbsm_to_mat: array `rowind' not allocated");
+err3:    {  print("hbsm_to_mat: array `rowind' not allocated");
             goto fail;
          }
          if (hbsm->values == NULL)
-err4:    {  error("hbsm_to_mat: array `values' not allocated");
+err4:    {  print("hbsm_to_mat: array `values' not allocated");
             goto fail;
          }
          A = create_mat(hbsm->nrow, hbsm->ncol);
          for (j = 1; j <= hbsm->ncol; j++)
          for (k = hbsm->colptr[j]; k < hbsm->colptr[j+1]; k++)
          {  if (!(1 <= k && k <= hbsm->nnzero))
-err5:       {  error("hbsm_to_mat: invalid column pointer");
+err5:       {  print("hbsm_to_mat: invalid column pointer");
                goto fail;
             }
             i = hbsm->rowind[k];
             if (!(1 <= i && i <= hbsm->nrow))
-err6:       {  error("hbsm_to_mat: invalid row index");
+err6:       {  print("hbsm_to_mat: invalid row index");
                goto fail;
             }
             if (i < j)
-err7:       {  error("hbsm_to_mat: invalid matrix structure");
+err7:       {  print("hbsm_to_mat: invalid matrix structure");
                goto fail;
             }
             val = hbsm->values[k];
@@ -692,7 +690,7 @@ err7:       {  error("hbsm_to_mat: invalid matrix structure");
          }
       }
       else
-      {  error("hbsm_to_mat: can't convert matrix of type `%s'",
+      {  print("hbsm_to_mat: can't convert matrix of type `%s'",
             hbsm->mxtype);
          goto fail;
       }
