@@ -128,12 +128,12 @@ INV *inv_create(int m, int max_upd)
 -- has been eliminated and the factorization has been recomputed again
 -- using the routine inv_decomp. For details of obtaining information
 -- needed for repairing the basis matrix see the routine luf_decomp (in
--- the section GLPLUF).
+-- the module GLPLUF).
 --
 -- *Algorithm*
 --
 -- The routine inv_decomp is an interface to the routine luf_decomp
--- (see the section 'glpluf'), which actually computes LU-factorization
+-- (see the module 'glpluf'), which actually computes LU-factorization
 -- of the basis matrix B in the form
 --
 --    [B] = (F, V, P, Q),
@@ -441,7 +441,7 @@ void inv_btran(INV *inv, double x[])
 -- the elements u[k2,k1], u[k2,k1+1], ..., u[k2,k2-1] using the diagonal
 -- elements u[k1,k1], u[k1+1,k1+1], ..., u[k2-1,k2-1] as pivots in the
 -- same way as described in comments to the routine luf_decomp (see the
--- section 'glpluf'). Note that actually all operations are performed on
+-- module 'glpluf'). Note that actually all operations are performed on
 -- the matrix V, not on the matrix U. During the elimination process the
 -- routine permutes neither rows nor columns, therefore only the k2-th
 -- row of the matrix U is changed.
