@@ -909,7 +909,7 @@ int ipm1(MAT *_A, double _b[], double _c[], double _x[], double _y[],
          /* compute the next point using Mehrotra's predictor-corrector
             technique */
          if (make_step())
-         {  print("NUMERICAL UNSTABILITY; OPTIMIZATION TERMINATED");
+         {  print("NUMERICAL INSTABILITY; OPTIMIZATION TERMINATED");
             status = 4;
             break;
          }
