@@ -29,8 +29,6 @@
 #include "glplang.h"
 #include "glpmat.h"
 
-#define error print
-
 /*----------------------------------------------------------------------
 -- gener_name - generate plain row/column name.
 --
@@ -303,7 +301,7 @@ struct elem *build_form(struct prob *prob, int i)
                }
                else
 err:           {  /* both multiplicands are linear form */
-                  error("build_form: non-linear constraint `%s' detecte"
+                  print("build_form: non-linear constraint `%s' detecte"
                      "d", gener_name(prob, i));
                   return NULL;
                }
@@ -316,7 +314,7 @@ err:           {  /* both multiplicands are linear form */
                if (!(y->j == 0 && y->next == NULL)) goto err;
                /* the divisor should be non-zero */
                if (y->val == 0.0)
-               {  error("build_form: zero divide occured on processing "
+               {  print("build_form: zero divide occured on processing "
                      "constraint `%s'", gener_name(prob, i));
                   return NULL;
                }
@@ -372,7 +370,7 @@ int gener_lp(char *fname)
       /* open the output text file */
       fp = fopen(fname, "w");
       if (fp == NULL)
-      {  error("gener_lp: can't create `%s' - %s", fname,
+      {  print("gener_lp: can't create `%s' - %s", fname,
             strerror(errno));
          goto fail;
       }
@@ -489,7 +487,7 @@ int gener_lp(char *fname)
       fprintf(fp, "End of output\n");
       fflush(fp);
       if (ferror(fp))
-      {  error("gener_lp: can't write to `%s' - %s", fname,
+      {  print("gener_lp: can't write to `%s' - %s", fname,
             strerror(errno));
          goto fail;
       }
@@ -499,7 +497,7 @@ int gener_lp(char *fname)
       ufree(rhs);
       return 0;
 fail: /* an error occurred */
-      error("gener_lp: generating impossible due to error");
+      print("gener_lp: generating impossible due to error");
       if (fp != NULL) fclose(fp);
       if (prob != NULL) delete_prob(prob);
       if (A != NULL) delete_mat(A);
