@@ -70,7 +70,7 @@ int init_lib_env(void)
       env->mem_tpeak = 0;
       env->mem_count = 0;
       env->mem_cpeak = 0;
-      env->last_watch = -DBL_MAX;
+      env->start = gmt2000();
       /* save a pointer to the environmental block */
       save_pointer(env);
       /* initialization completed */
