@@ -122,6 +122,78 @@ void _insist(char *expr, char *file, int line)
       /* no return */
 }
 
+/*----------------------------------------------------------------------
+-- jday - convert calendar date to Julian day.
+--
+-- This procedure converts a calendar date, Gregorian calendar, to the
+-- corresponding Julian day number j. From the given day d, month m, and
+-- year y, the Julian day number j is computed without using tables. The
+-- procedure is valid for any valid Gregorian calendar date. */
+
+static int jday(int d, int m, int y)
+{     int c, ya, j;
+      if (m > 2) m -= 3; else m += 9, y--;
+      c = y / 100;
+      ya = y - 100 * c;
+      j = (146097 * c) / 4 + (1461 * ya) / 4 + (153 * m + 2) / 5 + d +
+         1721119;
+      return j;
+}
+
+/*----------------------------------------------------------------------
+-- gmt2000 - determine the current time.
+--
+-- *Synopsis*
+--
+-- #include "glplib.h"
+-- double gmt2000(void);
+--
+-- *Returns*
+--
+-- The routine gmt2000 returns the current time, in seconds, elapsed
+-- since 12:00:00 GMT January 1, 2000. */
+
+double gmt2000(void)
+{     /* this is a platform independent version */
+      time_t timer;
+      struct tm *tm;
+      int j2000, j;
+      double secs;
+      timer = time(NULL);
+      tm = gmtime(&timer);
+      j2000 = 2451545 /* = jday(1, 1, 2000) */;
+      j = jday(tm->tm_mday, tm->tm_mon + 1, 1900 + tm->tm_year);
+      secs = (((double)(j - j2000) * 24.0 + (double)tm->tm_hour) * 60.0
+         + (double)tm->tm_min) * 60.0 + (double)tm->tm_sec - 43200.0;
+      return secs;
+}
+
+#if 0
+double gmt2000(void)
+{     /* this is a version for Win32 */
+      SYSTEMTIME st;
+      FILETIME ft0, ft;
+      double secs;
+      /* ft0 = 12:00:00 GMT January 1, 2000 */
+      ft0.dwLowDateTime  = 0xBAA22000;
+      ft0.dwHighDateTime = 0x01BF544F;
+      GetSystemTime(&st);
+      SystemTimeToFileTime(&st, &ft);
+      /* ft = ft - ft0 */
+      if (ft.dwLowDateTime >= ft0.dwLowDateTime)
+      {  ft.dwLowDateTime  -= ft0.dwLowDateTime;
+         ft.dwHighDateTime -= ft0.dwHighDateTime;
+      }
+      else
+      {  ft.dwLowDateTime  += (0xFFFFFFFF - ft0.dwLowDateTime) + 1;
+         ft.dwHighDateTime -= ft0.dwHighDateTime + 1;
+      }
+      secs = (4294967296.0 * (double)(LONG)ft.dwHighDateTime +
+         (double)ft.dwLowDateTime) / 10000000.0;
+      return secs;
+}
+#endif
+
 /*----------------------------------------------------------------------
 -- watch - take reading of stop-watch.
 --
@@ -132,16 +204,12 @@ void _insist(char *expr, char *file, int line)
 --
 -- *Returns*
 --
--- The routine watch returns the processor time in seconds. */
+-- The routine watch returns the time, in seconds, elapsed since the
+-- program started. */
 
 double watch(void)
 {     ENV *env = get_env_ptr();
-      double value;
-      value = (double)clock() / (double)CLOCKS_PER_SEC;
-      if (value < env->last_watch - 1.0)
-         print("watch: *** stop-watch overflow ***");
-      env->last_watch = value;
-      return value;
+      return gmt2000() - env->start;
 }
 
 /* eof */
