@@ -410,8 +410,8 @@ static int prepro_row(int i, double tol)
          }
       }
       else
-      {  /* if f_min[i] > lb[i] + eps, the lower bound of the i-th row is
-            redundant and therefore may be removed */
+      {  /* if f_min[i] > lb[i] + eps, the lower bound of the i-th row
+            is redundant and therefore may be removed */
          lp->lb[i] = -DBL_MAX;
       }
       /* analyze f_max[i] */
@@ -431,8 +431,8 @@ static int prepro_row(int i, double tol)
          }
       }
       else
-      {  /* if f_max[i] < ub[i] + eps, the upper bound of the i-th row is
-            redundant and therefore may be removed */
+      {  /* if f_max[i] < ub[i] + eps, the upper bound of the i-th row
+            is redundant and therefore may be removed */
          lp->ub[i] = +DBL_MAX;
       }
       return 0;
