@@ -56,6 +56,7 @@ LPX *lpx_create_prob(void)
       lp->m = lp->n = 0;
       lp->pool = create_str_pool();
       lp->buf = ucalloc(255+1, sizeof(char));
+      lp->class = LPX_LP;
       lp->name = ucalloc(1+m_max+n_max, sizeof(STR *));
       lp->name[0] = NULL;
       lp->typx = ucalloc(1+m_max+n_max, sizeof(int));
@@ -77,8 +78,9 @@ LPX *lpx_create_prob(void)
       lp->bbar = ucalloc(1+m_max, sizeof(double));
       lp->pi = ucalloc(1+m_max, sizeof(double));
       lp->cbar = ucalloc(1+n_max, sizeof(double));
-      lp->mip = 0;
       lp->kind = NULL;
+      lp->i_stat = LPX_I_UNDEF;
+      lp->mipx = NULL;
       lpx_reset_parms(lp);
       return lp;
 }
@@ -125,7 +127,10 @@ void lpx_realloc_prob(LPX *lp, int m_max, int n_max)
       reallocate(double, lp->bbar, 1+m_max, 1+m);
       reallocate(double, lp->pi, 1+m_max, 1+m);
       reallocate(double, lp->cbar, 1+n_max, 1+n);
-      if (lp->mip) reallocate(int, lp->kind, 1+n_max, 1+n);
+      if (lp->class == LPX_MIP)
+      {  reallocate(int, lp->kind, 1+n_max, 1+n);
+         reallocate(double, lp->mipx, 1+m_max+n_max, 1+m+n);
+      }
 #     undef reallocate
       lp->m_max = m_max;
       lp->n_max = n_max;
@@ -213,6 +218,7 @@ void lpx_add_rows(LPX *lp, int nrs)
       lp->b_stat = LPX_B_UNDEF;
       lp->p_stat = LPX_P_UNDEF;
       lp->d_stat = LPX_D_UNDEF;
+      lp->i_stat = LPX_I_UNDEF;
       return;
 }
 
@@ -237,6 +243,7 @@ void lpx_add_rows(LPX *lp, int nrs)
 void lpx_add_cols(LPX *lp, int ncs)
 {     int m = lp->m;
       int n = lp->n;
+      int class = lp->class;
       STR **name = lp->name;
       int *typx = lp->typx;
       double *lb = lp->lb;
@@ -245,7 +252,6 @@ void lpx_add_cols(LPX *lp, int ncs)
       int *mark = lp->mark;
       double *coef = lp->coef;
       int *tagx = lp->tagx;
-      int mip = lp->mip;
       int *kind = lp->kind;
       int n_new, j;
       if (ncs < 1)
@@ -282,7 +288,7 @@ void lpx_add_cols(LPX *lp, int ncs)
          /* the new column is non-basic */
          tagx[j] = LPX_NS;
          /* the new column is continuous */
-         if (mip) kind[j-m] = LPX_CV;
+         if (class == LPX_MIP) kind[j-m] = LPX_CV;
       }
       /* set new number of columns */
       lp->n = n_new;
@@ -292,6 +298,7 @@ void lpx_add_cols(LPX *lp, int ncs)
       lp->b_stat = LPX_B_UNDEF;
       lp->p_stat = LPX_P_UNDEF;
       lp->d_stat = LPX_D_UNDEF;
+      lp->i_stat = LPX_I_UNDEF;
       return;
 }
 
@@ -490,6 +497,7 @@ void lpx_set_row_bnds(LPX *lp, int i, int typx, double lb, double ub)
       /* invalidate the basic solution */
       lp->p_stat = LPX_P_UNDEF;
       lp->d_stat = LPX_D_UNDEF;
+      lp->i_stat = LPX_I_UNDEF;
       return;
 }
 
@@ -560,6 +568,52 @@ void lpx_set_col_bnds(LPX *lp, int j, int typx, double lb, double ub)
       /* invalidate the basic solution */
       lp->p_stat = LPX_P_UNDEF;
       lp->d_stat = LPX_D_UNDEF;
+      lp->i_stat = LPX_I_UNDEF;
+      return;
+}
+
+/*----------------------------------------------------------------------
+-- lpx_set_class - set (change) problem class.
+--
+-- *Synopsis*
+--
+-- #include "glplpx.h"
+-- void lpx_set_class(LPX *lp, int class);
+--
+-- *Description*
+--
+-- The routine lpx_set_class sets (changes) the class of the problem
+-- object as specified by the parameter class:
+--
+-- LPX_LP  - pure linear programming (LP) problem;
+-- LPX_MIP - mixed integer programming (MIP) problem. */
+
+void lpx_set_class(LPX *lp, int class)
+{     switch (class)
+      {  case LPX_LP:
+            if (lp->class == LPX_MIP)
+            {  /* deallocate MIP data segment */
+               ufree(lp->kind), lp->kind = NULL;
+               ufree(lp->mipx), lp->mipx = NULL;
+            }
+            break;
+         case LPX_MIP:
+            if (lp->class == LPX_LP)
+            {  /* allocate and initialize MIP data segment */
+               int j;
+               lp->kind = ucalloc(1+lp->n_max, sizeof(int));
+               lp->mipx = ucalloc(1+lp->m_max+lp->n_max, sizeof(double))
+                  ;
+               for (j = 1; j <= lp->n; j++) lp->kind[j] = LPX_CV;
+               lp->i_stat = LPX_I_UNDEF;
+            }
+            break;
+         default:
+            fault("lpx_set_class: class = %d; invalid parameter",
+               class);
+      }
+      /* set (change) problem class */
+      lp->class = class;
       return;
 }
 
@@ -580,18 +634,15 @@ void lpx_set_col_bnds(LPX *lp, int j, int typx, double lb, double ub)
 -- LPX_IV - integer variable. */
 
 void lpx_set_col_kind(LPX *lp, int j, int kind)
-{     if (!(1 <= j && j <= lp->n))
+{     if (lp->class != LPX_MIP)
+         fault("lpx_set_col_kind: error -- not a MIP problem");
+      if (!(1 <= j && j <= lp->n))
          fault("lpx_set_col_kind: j = %d; column number out of range",
             j);
       if (!(kind == LPX_CV || kind == LPX_IV))
          fault("lpx_set_col_kind: kind = %d; invalid parameter", kind);
-      if (!lp->mip)
-      {  int jj;
-         lp->mip = 1;
-         lp->kind = ucalloc(1+lp->n_max, sizeof(int));
-         for (jj = 1; jj <= lp->n; jj++) lp->kind[jj] = LPX_CV;
-      }
       lp->kind[j] = kind;
+      lp->i_stat = LPX_I_UNDEF;
       return;
 }
 
@@ -619,6 +670,7 @@ void lpx_set_obj_dir(LPX *lp, int dir)
       /* invalidate the basic solution */
       lp->p_stat = LPX_P_UNDEF;
       lp->d_stat = LPX_D_UNDEF;
+      lp->i_stat = LPX_I_UNDEF;
       return;
 }
 
@@ -661,6 +713,7 @@ void lpx_set_row_coef(LPX *lp, int i, double coef)
       /* invalidate the basic solution */
       lp->p_stat = LPX_P_UNDEF;
       lp->d_stat = LPX_D_UNDEF;
+      lp->i_stat = LPX_I_UNDEF;
       return;
 }
 
@@ -686,6 +739,7 @@ void lpx_set_col_coef(LPX *lp, int j, double coef)
       /* invalidate the basic solution */
       lp->p_stat = LPX_P_UNDEF;
       lp->d_stat = LPX_D_UNDEF;
+      lp->i_stat = LPX_I_UNDEF;
       return;
 }
 
@@ -832,6 +886,7 @@ void lpx_set_col_stat(LPX *lp, int j, int stat)
 -- "rewind" the file in order to begin enumerating again from the first
 -- element. */
 
+#if 0
 void lpx_load_mat(LPX *lp,
       void *info, double (*mat)(void *info, int *i, int *j))
 {     spm_load_data(lp->A, info, mat);
@@ -839,6 +894,109 @@ void lpx_load_mat(LPX *lp,
       lp->b_stat = LPX_B_UNDEF;
       lp->p_stat = LPX_P_UNDEF;
       lp->d_stat = LPX_D_UNDEF;
+      lp->i_stat = LPX_I_UNDEF;
+      return;
+}
+#else
+struct mat_info
+{     /* transit information passed to the routine mat */
+      LPX *lp;
+      void *info;
+      double (*mat)(void *info, int *i, int *j);
+};
+
+static double mat(void *_info, int *i, int *j)
+{     /* this intermediate routine is used to scale data loaded in the
+         constraint matrix */
+      struct mat_info *info = _info;
+      LPX *lp = info->lp;
+      int m = lp->m;
+      int n = lp->n;
+      double aij = info->mat(info->info, i, j);
+      if (!(*i == 0 && *j == 0))
+      {  if (!(1 <= *i && *i <= m))
+            fault("lpx_load_mat: i = %d; invalid row number", *i);
+         if (!(1 <= *j && *j <= n))
+            fault("lpx_load_mat: j = %d; invalid column number", *j);
+         if (aij == 0.0)
+            fault("lpx_load_mat: i = %d, j = %d; zero coefficient not a"
+               "llowed", *i, *j);
+         aij *= (lp->rs[*i] * lp->rs[m + *j]);
+      }
+      return aij;
+}
+
+void lpx_load_mat(LPX *lp,
+      void *_info, double (*_mat)(void *info, int *i, int *j))
+{     struct mat_info info;
+      /* load scaled data in the constraint matrix */
+      info.lp = lp;
+      info.info = _info;
+      info.mat = _mat;
+      spm_load_data(lp->A, &info, mat);
+      /* invalidate the factorization and the basic solution */
+      lp->b_stat = LPX_B_UNDEF;
+      lp->p_stat = LPX_P_UNDEF;
+      lp->d_stat = LPX_D_UNDEF;
+      lp->i_stat = LPX_I_UNDEF;
+      return;
+}
+#endif
+
+/*----------------------------------------------------------------------
+-- lpx_load_mat3 - load constraint matrix.
+--
+-- *Synopsis*
+--
+-- #include "glplpx.h"
+-- void lpx_load_mat3(LPX *lp, int nz, int rn[], int cn[], double a[]);
+--
+-- *Description*
+--
+-- The routine lpx_load_mat3 loads non-zero elements of the constraint
+-- matrix (i.e. constraint coefficients) from the arrays rn, cn, and a.
+-- All existing contents of the constraint matrix is destroyed.
+--
+-- The parameter nz specifies number of loaded non-zero coefficients.
+--
+-- A particular constraint coefficient should be specified as a triplet
+-- (rn[k], cn[k], a[k]), k = 1, ..., nz, where rn[k] is row index, cn[k]
+-- is column index, and a[k] is its numerical value. Coefficients may be
+-- enumerated in arbitrary order. Note that zero coefficients as well as
+-- multiplets (i.e. coefficients with identical row and column indices)
+-- are not allowed. */
+
+struct mat3_info
+{     /* transit information passed to the routine mat3 */
+      int nz;
+      int *rn;
+      int *cn;
+      double *a;
+      int ptr;
+};
+
+static double mat3(void *_info, int *i, int *j)
+{     /* "reads" a next element of the constraint matrix */
+      struct mat3_info *info = _info;
+      info->ptr++;
+      if (info->ptr > info->nz)
+      {  *i = *j = 0;
+         info->ptr = 0;
+         return 0.0;
+      }
+      *i = info->rn[info->ptr];
+      *j = info->cn[info->ptr];
+      return info->a[info->ptr];
+}
+
+void lpx_load_mat3(LPX *lp, int nz, int rn[], int cn[], double a[])
+{     struct mat3_info info;
+      info.nz = nz;
+      info.rn = rn;
+      info.cn = cn;
+      info.a = a;
+      info.ptr = 0;
+      lpx_load_mat(lp, &info, mat3);
       return;
 }
 
@@ -900,6 +1058,7 @@ void lpx_set_mat_row(LPX *lp, int i, int len, int ndx[], double val[])
       /* invalidate the basic solution */
       lp->p_stat = LPX_P_UNDEF;
       lp->d_stat = LPX_D_UNDEF;
+      lp->i_stat = LPX_I_UNDEF;
       return;
 }
 
@@ -941,6 +1100,7 @@ void lpx_set_mat_col(LPX *lp, int j, int len, int ndx[], double val[])
       /* invalidate the basic solution */
       lp->p_stat = LPX_P_UNDEF;
       lp->d_stat = LPX_D_UNDEF;
+      lp->i_stat = LPX_I_UNDEF;
       return;
 }
 
@@ -976,7 +1136,7 @@ void lpx_unmark_all(LPX *lp)
 --
 -- *Description*
 --
--- The routine lpx_mark_row assigns an integer mark the i-th row.
+-- The routine lpx_mark_row assigns an integer mark to the i-th row.
 --
 -- The sense of marking depends on what operation will be performed on
 -- the LP problem object. */
@@ -1054,6 +1214,7 @@ skip: /* clear marked rows of the constraint matrix */
       /* invalidate the basic solution */
       lp->p_stat = LPX_P_UNDEF;
       lp->d_stat = LPX_D_UNDEF;
+      lp->i_stat = LPX_I_UNDEF;
       return;
 }
 
@@ -1073,6 +1234,7 @@ skip: /* clear marked rows of the constraint matrix */
 void lpx_del_items(LPX *lp)
 {     int m = lp->m;
       int n = lp->n;
+      int class = lp->class;
       STR **name = lp->name;
       int *typx = lp->typx;
       double *lb = lp->lb;
@@ -1081,7 +1243,6 @@ void lpx_del_items(LPX *lp)
       int *mark = lp->mark;
       double *coef = lp->coef;
       int *tagx = lp->tagx;
-      int mip = lp->mip;
       int *kind = lp->kind;
       int m_new, n_new, k, k_new;
       /* delete marked rows and columns from the LP problem object */
@@ -1102,7 +1263,7 @@ void lpx_del_items(LPX *lp)
             rs[k_new] = rs[k];
             coef[k_new] = coef[k];
             tagx[k_new] = tagx[k];
-            if (mip && k > m) kind[n_new] = kind[k-m];
+            if (class == LPX_MIP && k > m) kind[n_new] = kind[k-m];
          }
       }
       /* delete marked rows from the constraint matrix */
@@ -1118,6 +1279,7 @@ void lpx_del_items(LPX *lp)
       lp->b_stat = LPX_B_UNDEF;
       lp->p_stat = LPX_P_UNDEF;
       lp->d_stat = LPX_D_UNDEF;
+      lp->i_stat = LPX_I_UNDEF;
       return;
 }
 
@@ -1152,7 +1314,10 @@ void lpx_delete_prob(LPX *lp)
       ufree(lp->bbar);
       ufree(lp->pi);
       ufree(lp->cbar);
-      if (lp->mip) ufree(lp->kind);
+      if (lp->class == LPX_MIP)
+      {  ufree(lp->kind);
+         ufree(lp->mipx);
+      }
       ufree(lp);
       return;
 }
