@@ -65,6 +65,58 @@ int lpx_get_num_cols(LPX *lp)
       return n;
 }
 
+/*----------------------------------------------------------------------
+-- lpx_get_num_int - determine number of integer columns.
+--
+-- *Synopsis*
+--
+-- #include "glplpx.h"
+-- int lpx_get_num_int(LPX *lp);
+--
+-- *Returns*
+--
+-- The routine lpx_get_num_int returns number of columns (structural
+-- variables) in the problem object, which are marked as integer. */
+
+int lpx_get_num_int(LPX *lp)
+{     int count = 0, j;
+      if (lp->class != LPX_MIP)
+         fault("lpx_get_num_int: error -- not a MIP problem");
+      for (j = 1; j <= lp->n; j++)
+         if (lp->kind[j] == LPX_IV) count++;
+      return count;
+}
+
+/*----------------------------------------------------------------------
+-- lpx_get_num_bin - determine number of binary columns.
+--
+-- *Synopsis*
+--
+-- #include "glplpx.h"
+-- int lpx_get_num_bin(LPX *lp);
+--
+-- *Returns*
+--
+-- The routine lpx_get_num_bin returns number of columns (structural
+-- variables) in the problem object, which are marked as integer and
+-- have zero lower bound and unity upper bound. */
+
+int lpx_get_num_bin(LPX *lp)
+{     int count = 0, j, k;
+      double trick = 1e-12;
+      if (lp->class != LPX_MIP)
+         fault("lpx_get_num_bin: error -- not a MIP problem");
+      for (j = 1; j <= lp->n; j++)
+      {  if (lp->kind[j] == LPX_IV)
+         {  k = lp->m + j;
+            if (lp->typx[k] == LPX_DB &&
+                fabs(lp->lb[k] * lp->rs[k])       <= trick &&
+                fabs(lp->ub[k] * lp->rs[k] - 1.0) <= trick) count++;
+         }
+      }
+      return count;
+}
+
 /*----------------------------------------------------------------------
 -- lpx_get_num_nz - determine number of constraint coefficients.
 --
@@ -238,6 +290,27 @@ void lpx_get_col_bnds(LPX *lp, int j, int *typx, double *lb, double *ub)
       return;
 }
 
+/*----------------------------------------------------------------------
+-- lpx_get_class - query problem class.
+--
+-- *Synopsis*
+--
+-- #include "glplpx.h"
+-- int lpx_get_class(LPX *lp);
+--
+-- *Returns*
+--
+-- The routine lpx_get_class returns a problem class for the specified
+-- problem object:
+--
+-- LPX_LP  - pure linear programming (LP) problem;
+-- LPX_MIP - mixed integer programming (MIP) problem. */
+
+int lpx_get_class(LPX *lp)
+{     int class = lp->class;
+      return class;
+}
+
 /*----------------------------------------------------------------------
 -- lpx_get_col_kind - query column kind.
 --
@@ -255,10 +328,12 @@ void lpx_get_col_bnds(LPX *lp, int j, int *typx, double *lb, double *ub)
 -- LPX_IV - integer variable. */
 
 int lpx_get_col_kind(LPX *lp, int j)
-{     if (!(1 <= j && j <= lp->n))
+{     if (lp->class != LPX_MIP)
+         fault("lpx_get_col_kind: error -- not a MIP problem");
+      if (!(1 <= j && j <= lp->n))
          fault("lpx_get_col_kind: j = %d; column number out of range",
             j);
-      return (lp->mip ? lp->kind[j] : LPX_CV);
+      return lp->kind[j];
 }
 
 /*----------------------------------------------------------------------
@@ -546,14 +621,14 @@ int lpx_get_dual_stat(LPX *lp)
 -- *Synopsis*
 --
 -- #include "glplpx.h"
--- void lpx_get_row_info(LPX *lp, int i, int *tagx, double *valx,
+-- void lpx_get_row_info(LPX *lp, int i, int *tagx, double *vx,
 --    double *dx);
 --
 -- *Description*
 --
 -- The routine lpx_get_row_info stores status, primal value and dual
 -- value of the i-th auxiliary variable (row) to locations, which the
--- parameters tagx, valx, and dx point to, respectively.
+-- parameters tagx, vx, and dx point to, respectively.
 --
 -- The status code has the following meaning:
 --
@@ -563,71 +638,53 @@ int lpx_get_dual_stat(LPX *lp)
 -- LPX_NF - non-basic free (unbounded) variable;
 -- LPX_NS - non-basic fixed variable.
 --
--- If some of pointers tagx, valx, or dx is NULL, the corresponding
--- value is not stored. */
+-- If some of pointers tagx, vx, or dx is NULL, the corresponding value
+-- is not stored. */
 
-void lpx_get_row_info(LPX *lp, int i, int *tagx, double *valx,
-      double *dx)
+void lpx_get_row_info(LPX *lp, int i, int *tagx, double *vx, double *dx)
 {     int m = lp->m;
       int n = lp->n;
-      int typx_i, tagx_i, t;
-      double lb_i, ub_i, valx_i, dx_i, temp;
+      int tagx_i, t;
+      double vx_i, dx_i;
       if (!(1 <= i && i <= m))
          fault("lpx_get_row_info: i = %d; row number out of range", i);
       /* obtain the status */
       tagx_i = lp->tagx[i];
       if (tagx != NULL) *tagx = tagx_i;
       /* obtain the primal value */
-      if (valx != NULL)
+      if (vx != NULL)
       {  if (lp->p_stat == LPX_P_UNDEF)
          {  /* the primal value is undefined */
-            valx_i = 0.0;
+            vx_i = 0.0;
          }
          else
          {  if (tagx_i == LPX_BS)
             {  /* basic variable */
                t = lp->posx[i]; /* x[i] = xB[t] */
                insist(1 <= t && t <= m);
-               valx_i = lp->bbar[t];
+               vx_i = lp->bbar[t];
                /* round the primal value (if required) */
-               if (lp->round)
-               {  typx_i = lp->typx[i];
-                  if (typx_i == LPX_LO || typx_i == LPX_DB ||
-                      typx_i == LPX_FX)
-                  {  /* x[i] has lower bound */
-                     lb_i = lp->lb[i];
-                     temp = fabs(valx_i - lb_i) / (1.0 + fabs(lb_i));
-                     if (temp < lp->tol_bnd) valx_i = lb_i;
-                  }
-                  if (typx_i == LPX_UP || typx_i == LPX_DB ||
-                      typx_i == LPX_FX)
-                  {  /* x[i] has upper bound */
-                     ub_i = lp->ub[i];
-                     temp = fabs(valx_i - ub_i) / (1.0 + fabs(ub_i));
-                     if (temp < lp->tol_bnd) valx_i = ub_i;
-                  }
-                  if (fabs(valx_i) < lp->tol_bnd) valx_i = 0.0;
-               }
+               if (lp->round && fabs(vx_i) <= lp->tol_bnd) vx_i = 0.0;
             }
             else
             {  /* non-basic variable */
                switch (tagx_i)
                {  case LPX_NL:
-                     valx_i = lp->lb[i]; break;
+                     vx_i = lp->lb[i]; break;
                   case LPX_NU:
-                     valx_i = lp->ub[i]; break;
+                     vx_i = lp->ub[i]; break;
                   case LPX_NF:
-                     valx_i = 0.0; break;
+                     vx_i = 0.0; break;
                   case LPX_NS:
-                     valx_i = lp->lb[i]; break;
+                     vx_i = lp->lb[i]; break;
                   default:
                      insist(tagx_i != tagx_i);
                }
             }
             /* unscale the primal value */
-            valx_i /= lp->rs[i];
+            vx_i /= lp->rs[i];
          }
-         *valx = valx_i;
+         *vx = vx_i;
       }
       /* obtain the dual value */
       if (dx != NULL)
@@ -646,7 +703,7 @@ void lpx_get_row_info(LPX *lp, int i, int *tagx, double *valx,
                insist(1 <= t && t <= n);
                dx_i = lp->cbar[t];
                /* round the dual value (if required) */
-               if (lp->round && fabs(dx_i) < lp->tol_dj) dx_i = 0.0;
+               if (lp->round && fabs(dx_i) <= lp->tol_dj) dx_i = 0.0;
             }
             /* unscale the dual value */
             dx_i *= lp->rs[i];
@@ -662,14 +719,14 @@ void lpx_get_row_info(LPX *lp, int i, int *tagx, double *valx,
 -- *Synopsis*
 --
 -- #include "glplpx.h"
--- void lpx_get_col_info(LPX *lp, int j, int *tagx, double *valx,
+-- void lpx_get_col_info(LPX *lp, int j, int *tagx, double *vx,
 --    double *dx);
 --
 -- *Description*
 --
 -- The routine lpx_get_col_info stores status, primal value and dual
 -- value of the j-th structural variable (column) to locations, which
--- the parameters tagx, valx, and dx point to, respectively.
+-- the parameters tagx, vx, and dx point to, respectively.
 --
 -- The status code has the following meaning:
 --
@@ -679,15 +736,14 @@ void lpx_get_row_info(LPX *lp, int i, int *tagx, double *valx,
 -- LPX_NF - non-basic free (unbounded) variable;
 -- LPX_NS - non-basic fixed variable.
 --
--- If some of pointers tagx, valx, or dx is NULL, the corresponding
--- value is not stored. */
+-- If some of pointers tagx, vx, or dx is NULL, the corresponding value
+-- is not stored. */
 
-void lpx_get_col_info(LPX *lp, int j, int *tagx, double *valx,
-      double *dx)
+void lpx_get_col_info(LPX *lp, int j, int *tagx, double *vx, double *dx)
 {     int m = lp->m;
       int n = lp->n;
-      int typx_j, tagx_j, t;
-      double lb_j, ub_j, valx_j, dx_j, temp;
+      int tagx_j, t;
+      double vx_j, dx_j;
       if (!(1 <= j && j <= n))
          fault("lpx_get_col_info: j = %d; column number out of range",
             j);
@@ -696,56 +752,39 @@ void lpx_get_col_info(LPX *lp, int j, int *tagx, double *valx,
       tagx_j = lp->tagx[j];
       if (tagx != NULL) *tagx = tagx_j;
       /* obtain the primal value */
-      if (valx != NULL)
+      if (vx != NULL)
       {  if (lp->p_stat == LPX_P_UNDEF)
          {  /* the primal value is undefined */
-            valx_j = 0.0;
+            vx_j = 0.0;
          }
          else
          {  if (tagx_j == LPX_BS)
             {  /* basic variable */
                t = lp->posx[j]; /* x[j] = xB[t] */
                insist(1 <= t && t <= m);
-               valx_j = lp->bbar[t];
+               vx_j = lp->bbar[t];
                /* round the primal value (if required) */
-               if (lp->round)
-               {  typx_j = lp->typx[j];
-                  if (typx_j == LPX_LO || typx_j == LPX_DB ||
-                      typx_j == LPX_FX)
-                  {  /* x[j] has lower bound */
-                     lb_j = lp->lb[j];
-                     temp = fabs(valx_j - lb_j) / (1.0 + fabs(lb_j));
-                     if (temp < lp->tol_bnd) valx_j = lb_j;
-                  }
-                  if (typx_j == LPX_UP || typx_j == LPX_DB ||
-                      typx_j == LPX_FX)
-                  {  /* x[j] has upper bound */
-                     ub_j = lp->ub[j];
-                     temp = fabs(valx_j - ub_j) / (1.0 + fabs(ub_j));
-                     if (temp < lp->tol_bnd) valx_j = ub_j;
-                  }
-                  if (fabs(valx_j) < lp->tol_bnd) valx_j = 0.0;
-               }
+               if (lp->round && fabs(vx_j) <= lp->tol_bnd) vx_j = 0.0;
             }
             else
             {  /* non-basic variable */
                switch (tagx_j)
                {  case LPX_NL:
-                     valx_j = lp->lb[j]; break;
+                     vx_j = lp->lb[j]; break;
                   case LPX_NU:
-                     valx_j = lp->ub[j]; break;
+                     vx_j = lp->ub[j]; break;
                   case LPX_NF:
-                     valx_j = 0.0; break;
+                     vx_j = 0.0; break;
                   case LPX_NS:
-                     valx_j = lp->lb[j]; break;
+                     vx_j = lp->lb[j]; break;
                   default:
                      insist(tagx_j != tagx_j);
                }
             }
             /* unscale the primal value */
-            valx_j *= lp->rs[j];
+            vx_j *= lp->rs[j];
          }
-         *valx = valx_j;
+         *vx = vx_j;
       }
       /* obtain the dual value */
       if (dx != NULL)
@@ -764,7 +803,7 @@ void lpx_get_col_info(LPX *lp, int j, int *tagx, double *valx,
                insist(1 <= t && t <= n);
                dx_j = lp->cbar[t];
                /* round the dual value (if required) */
-               if (lp->round && fabs(dx_j) < lp->tol_dj) dx_j = 0.0;
+               if (lp->round && fabs(dx_j) <= lp->tol_dj) dx_j = 0.0;
             }
             /* unscale the dual value */
             dx_j /= lp->rs[j];
@@ -784,188 +823,159 @@ void lpx_get_col_info(LPX *lp, int j, int *tagx, double *valx,
 --
 -- *Returns*
 --
--- The routine returns the current value of the objective function for
--- an LP problem object, which the parameter lp points to. */
+-- The routine lpx_get_obj_val returns the current value of the
+-- objective function for an LP problem object, which the parameter lp
+-- points to. */
 
 double lpx_get_obj_val(LPX *lp)
 {     int m = lp->m;
       int n = lp->n;
       int i, j;
-      double sum, coef, valx;
+      double sum, coef, vx;
       sum = lpx_get_obj_c0(lp);
       for (i = 1; i <= m; i++)
       {  coef = lpx_get_row_coef(lp, i);
          if (coef != 0.0)
-         {  lpx_get_row_info(lp, i, NULL, &valx, NULL);
-            sum += coef * valx;
+         {  lpx_get_row_info(lp, i, NULL, &vx, NULL);
+            sum += coef * vx;
          }
       }
       for (j = 1; j <= n; j++)
       {  coef = lpx_get_col_coef(lp, j);
          if (coef != 0.0)
-         {  lpx_get_col_info(lp, j, NULL, &valx, NULL);
-            sum += coef * valx;
+         {  lpx_get_col_info(lp, j, NULL, &vx, NULL);
+            sum += coef * vx;
          }
       }
       return sum;
 }
 
 /*----------------------------------------------------------------------
--- lpx_print_sol - write problem solution in printable format.
+-- lpx_get_mip_stat - query status of MIP solution.
 --
 -- *Synopsis*
 --
 -- #include "glplpx.h"
--- int lpx_print_sol(LPX *lp, char *fname);
---
--- *Description*
---
--- The routine lpx_print_sol writes the current basic solution of an LP
--- problem, which is specified by the pointer lp, to a text file, whose
--- name is the character string fname, in printable format.
---
--- Information reported by the routine lpx_print_sol is intended mainly
--- for visual analysis.
+-- int lpx_get_mip_stat(LPX *lp);
 --
 -- *Returns*
 --
--- If the operation was successful, the routine returns zero. Otherwise
--- the routine prints an error message and returns non-zero. */
+-- The routine lpx_get_mip_stat reports the status of a MIP solution
+-- found by the solver for a MIP problem object, which the parameter lp
+-- points to:
+--
+-- LPX_I_UNDEF  - the status is undefined (either the problem has not
+--                been solved yet or no integer feasible solution has
+--                been found yet).
+--
+-- LPX_I_OPT    - the solution is integer optimal.
+--
+-- LPX_I_FEAS   - the solution is integer feasible but its optimality
+--                (or non-optimality) has not been proven, perhaps due
+--                to premature termination of the search.
+--
+-- LPX_I_NOFEAS - the problem has no integer feasible solution (proven
+--                by the solver). */
 
-int lpx_print_sol(LPX *lp, char *fname)
-{     FILE *fp;
-      int what, round;
-      print("lpx_print_sol: writing problem solution to `%s'...",
-         fname);
-      fp = fopen(fname, "w");
-      if (fp == NULL)
-      {  print("lpx_print_sol: can't create `%s' - %s", fname,
-            strerror(errno));
-         goto fail;
-      }
-      /* problem name */
-      {  char *name;
-         name = lpx_get_prob_name(lp);
-         if (name == NULL) name = "";
-         fprintf(fp, "%-12s%s\n", "Problem:", name);
-      }
-      /* number of rows (auxiliary variables) */
-      {  int nr;
-         nr = lpx_get_num_rows(lp);
-         fprintf(fp, "%-12s%d\n", "Rows:", nr);
-      }
-      /* number of columns (structural variables) */
-      {  int nc;
-         nc = lpx_get_num_cols(lp);
-         fprintf(fp, "%-12s%d\n", "Columns:", nc);
-      }
-      /* number of non-zeros (constraint coefficients) */
-      {  int nz;
-         nz = lpx_get_num_nz(lp);
-         fprintf(fp, "%-12s%d\n", "Non-zeros:", nz);
-      }
-      /* solution status */
-      {  int status;
-         status = lpx_get_status(lp);
-         fprintf(fp, "%-12s%s\n", "Status:",
-            status == LPX_OPT    ? "OPTIMAL" :
-            status == LPX_FEAS   ? "FEASIBLE" :
-            status == LPX_INFEAS ? "INFEASIBLE (INTERMEDIATE)" :
-            status == LPX_NOFEAS ? "INFEASIBLE (FINAL)" :
-            status == LPX_UNBND  ? "UNBOUNDED" :
-            status == LPX_UNDEF  ? "UNDEFINED" : "???");
-      }
-      /* objective function */
-      {  int dir;
-         double obj;
-         dir = lpx_get_obj_dir(lp);
-         round = lp->round, lp->round = 1;
-         obj = lpx_get_obj_val(lp);
-         lp->round = round;
-         fprintf(fp, "%-12s%.6g %s\n", "Objective:", obj,
-            dir == LPX_MIN ? "(MINimization)" :
-            dir == LPX_MAX ? "(MAXimization)" : "(???)");
-      }
-      /* main sheet */
-      for (what = 1; what <= 2; what++)
-      {  int mn, ij;
-         fprintf(fp, "\n");
-         fprintf(fp, "  No. %-12s   St   Activity     Lower bound   Upp"
-            "er bound    Marginal\n",
-            what == 1 ? "  Row name" : "Column name");
-         fprintf(fp, "----- ------------   -- ------------- -----------"
-            "-- ------------- -------------\n");
-         mn = (what == 1 ? lpx_get_num_rows(lp) : lpx_get_num_cols(lp));
-         for (ij = 1; ij <= mn; ij++)
-         {  char *name;
-            int typx, tagx;
-            double lb, ub, valx, dx;
-            if (what == 1)
-            {  name = lpx_get_row_name(lp, ij);
-               if (name == NULL) name = "";
-               lpx_get_row_bnds(lp, ij, &typx, &lb, &ub);
-               round = lp->round, lp->round = 1;
-               lpx_get_row_info(lp, ij, &tagx, &valx, &dx);
-               lp->round = round;
-            }
-            else
-            {  name = lpx_get_col_name(lp, ij);
-               if (name == NULL) name = "";
-               lpx_get_col_bnds(lp, ij, &typx, &lb, &ub);
-               round = lp->round, lp->round = 1;
-               lpx_get_col_info(lp, ij, &tagx, &valx, &dx);
-               lp->round = round;
-            }
-            /* row/column ordinal number */
-            fprintf(fp, "%5d ", ij);
-            /* row column/name */
-            if (strlen(name) <= 12)
-               fprintf(fp, "%-12s ", name);
-            else
-               fprintf(fp, "%s\n%19s", name, "");
-            /* row/column status */
-            fprintf(fp, "  %s ",
-               tagx == LPX_BS ? "B " :
-               tagx == LPX_NL ? "NL" :
-               tagx == LPX_NU ? "NU" :
-               tagx == LPX_NF ? "NF" :
-               tagx == LPX_NS ? "NS" : "??");
-            /* row/column primal activity */
-            fprintf(fp, "%13.6g ", valx);
-            /* row/column lower bound */
-            if (typx == LPX_LO || typx == LPX_DB || typx == LPX_FX)
-               fprintf(fp, "%13.6g ", lb);
-            else
-               fprintf(fp, "%13s ", "");
-            /* row/column upper bound */
-            if (typx == LPX_UP || typx == LPX_DB)
-               fprintf(fp, "%13.6g ", ub);
-            else if (typx == LPX_FX)
-               fprintf(fp, "%13s ", "=");
-            else
-               fprintf(fp, "%13s ", "");
-            /* row/column dual activity */
-            if (tagx != LPX_BS)
-            {  if (dx == 0.0)
-                  fprintf(fp, "%13s", "< eps");
-               else
-                  fprintf(fp, "%13.6g", dx);
-            }
-            fprintf(fp, "\n");
+int lpx_get_mip_stat(LPX *lp)
+{     if (lp->class != LPX_MIP)
+         fault("lpx_get_mip_stat: error -- not a MIP problem");
+      return lp->i_stat;
+}
+
+/*----------------------------------------------------------------------
+-- lpx_get_mip_row - obtain row activity for MIP solution.
+--
+-- *Synopsis*
+--
+-- #include "glplpx.h"
+-- double lpx_get_mip_row(LPX *lp, int i);
+--
+-- *Returns*
+--
+-- The routine returns a value of the i-th auxiliary variable (row) for
+-- a MIP solution contained in the specified problem object. */
+
+double lpx_get_mip_row(LPX *lp, int i)
+{     double vx;
+      if (lp->class != LPX_MIP)
+         fault("lpx_get_mip_row: error -- not a MIP problem");
+      if (!(1 <= i && i <= lp->m))
+         fault("lpx_get_mip_row: i = %d; row number out of range", i);
+      if (!(lp->i_stat == LPX_I_OPT || lp->i_stat == LPX_I_FEAS))
+         vx = 0.0;
+      else
+      {  vx = lp->mipx[i];
+         if (lp->round)
+         {  double eps = lp->tol_bnd / lp->rs[i];
+            if (fabs(vx) <= eps) vx = 0.0;
          }
       }
-      fprintf(fp, "\n");
-      fprintf(fp, "End of output\n");
-      fflush(fp);
-      if (ferror(fp))
-      {  print("lpx_print_sol: can't write to `%s' - %s", fname,
-            strerror(errno));
-         goto fail;
+      return vx;
+}
+
+/*----------------------------------------------------------------------
+-- lpx_get_mip_col - obtain column activity for MIP solution.
+--
+-- *Synopsis*
+--
+-- #include "glplpx.h"
+-- double lpx_get_mip_col(LPX *lp, int j);
+--
+-- *Returns*
+--
+-- The routine returns a value of the j-th structural variable (column)
+-- for a MIP solution contained in the specified problem object. */
+
+double lpx_get_mip_col(LPX *lp, int j)
+{     double vx;
+      if (lp->class != LPX_MIP)
+         fault("lpx_get_mip_col: error -- not a MIP problem");
+      if (!(1 <= j && j <= lp->n))
+         fault("lpx_get_mip_col: j = %d; column number out of range",
+            j);
+      if (!(lp->i_stat == LPX_I_OPT || lp->i_stat == LPX_I_FEAS))
+         vx = 0.0;
+      else
+      {  vx = lp->mipx[lp->m+j];
+         if (lp->kind[j] == LPX_IV)
+            insist(vx == floor(vx));
+         else if (lp->round)
+         {  double eps = lp->tol_bnd * lp->rs[lp->m+j];
+            if (fabs(vx) <= eps) vx = 0.0;
+         }
       }
-      fclose(fp);
-      return 0;
-fail: if (fp != NULL) fclose(fp);
-      return 1;
+      return vx;
+}
+
+/*----------------------------------------------------------------------
+-- lpx_get_mip_obj - obtain value of the obj. func. for MIP solution.
+--
+-- *Synopsis*
+--
+-- #include "glplpx.h"
+-- double lpx_get_mip_obj(LPX *lp);
+--
+-- *Returns*
+--
+-- The routine lpx_get_mip_obj returns a value of the obj. function for
+-- a MIP solution contained in the specified problem object. */
+
+double lpx_get_mip_obj(LPX *lp)
+{     int i, j;
+      double sum, coef;
+      if (lp->class != LPX_MIP)
+         fault("lpx_get_mip_obj: error -- not a MIP problem");
+      sum = lpx_get_obj_c0(lp);
+      for (i = 1; i <= lp->m; i++)
+      {  coef = lpx_get_row_coef(lp, i);
+         if (coef != 0.0) sum += coef * lpx_get_mip_row(lp, i);
+      }
+      for (j = 1; j <= lp->n; j++)
+      {  coef = lpx_get_col_coef(lp, j);
+         if (coef != 0.0) sum += coef * lpx_get_mip_col(lp, j);
+      }
+      return sum;
 }
 
 /* eof */
