@@ -33,8 +33,6 @@
 #include "glpmps.h"
 #include "glpstr.h"
 
-#define error print
-
 /*----------------------------------------------------------------------
 -- load_mps - load linear programming model in MPS format.
 --
@@ -115,7 +113,7 @@ MPS *load_mps(char *_fname)
       print("load_mps: reading LP data from `%s'...", fname);
       fp = fopen(fname, "r");
       if (fp == NULL)
-      {  error("load_mps: unable to open `%s' - %s", fname,
+      {  print("load_mps: unable to open `%s' - %s", fname,
             strerror(errno));
          goto fail;
       }
@@ -131,7 +129,7 @@ MPS *load_mps(char *_fname)
       /* process NAME indicator card */
       if (read_card()) goto fail;
       if (memcmp(card, "NAME ", 5) != 0)
-      {  error("%s:%d: NAME indicator card missing", fname, seqn);
+      {  print("%s:%d: NAME indicator card missing", fname, seqn);
          goto fail;
       }
       memcpy(f3, card+14, 8); f3[8] = '\0'; strspx(f3);
@@ -142,7 +140,7 @@ MPS *load_mps(char *_fname)
       /* process ROWS section */
       if (read_card()) goto fail;
       if (memcmp(card, "ROWS ", 5) != 0)
-      {  error("%s:%d: ROWS indicator card missing", fname, seqn);
+      {  print("%s:%d: ROWS indicator card missing", fname, seqn);
          goto fail;
       }
       if (load_rows()) goto fail;
@@ -150,7 +148,7 @@ MPS *load_mps(char *_fname)
       print("load_mps: %d rows", mps->n_row);
       /* process COLUMNS section */
       if (memcmp(card, "COLUMNS ", 8) != 0)
-      {  error("%s:%d: COLUMNS indicator card missing", fname, seqn);
+      {  print("%s:%d: COLUMNS indicator card missing", fname, seqn);
          goto fail;
       }
       if (load_columns(t_col)) goto fail;
@@ -190,7 +188,7 @@ MPS *load_mps(char *_fname)
       }
       /* process ENDATA indicator card */
       if (memcmp(card, "ENDATA ", 7) != 0)
-      {  error("%s:%d: invalid indicator card", fname, seqn);
+      {  print("%s:%d: invalid indicator card", fname, seqn);
          goto fail;
       }
       print("load_mps: %d cards were read", seqn);
@@ -306,26 +304,26 @@ loop: seqn++;
       for (;;)
       {  c = fgetc(fp);
          if (ferror(fp))
-         {  error("%s:%d: read error - %s", fname, seqn,
+         {  print("%s:%d: read error - %s", fname, seqn,
                strerror(errno));
             return 1;
          }
          if (feof(fp))
          {  if (k == 0)
-               error("%s:%d: unexpected eof", fname, seqn);
+               print("%s:%d: unexpected eof", fname, seqn);
             else
-               error("%s:%d: missing final LF", fname, seqn);
+               print("%s:%d: missing final LF", fname, seqn);
             return 1;
          }
          if (c == '\r') continue;
          if (c == '\n') break;
          if (iscntrl(c))
-         {  error("%s:%d: invalid control character 0x%02X", fname,
+         {  print("%s:%d: invalid control character 0x%02X", fname,
                seqn, c);
             return 1;
          }
          if (k == 80)
-         {  error("%s:%d: card image too long", fname, seqn);
+         {  print("%s:%d: card image too long", fname, seqn);
             return 1;
          }
          card[k++] = (char)c;
@@ -345,7 +343,7 @@ loop: seqn++;
 static int split_card(void)
 {     /* col. 1: blank */
       if (memcmp(card+0, " ", 1))
-fail: {  error("%s:%d: invalid data card", fname, seqn);
+fail: {  print("%s:%d: invalid data card", fname, seqn);
          return 1;
       }
       /* col. 2-3: field 1 (code) */
@@ -400,31 +398,31 @@ loop: /* process the next data card */
       if (card[0] != ' ') goto done;
       if (split_card()) return 1;
       if (strcmp(f3, "'MARKER'") == 0)
-      {  error("%s:%d: invalid use of marker in ROWS section", fname,
+      {  print("%s:%d: invalid use of marker in ROWS section", fname,
             seqn);
          return 1;
       }
       if (f1[0] == '\0')
-      {  error("%s:%d: missing row type in field 1", fname, seqn);
+      {  print("%s:%d: missing row type in field 1", fname, seqn);
          return 1;
       }
       if (!(strchr("NGLE", f1[0]) != NULL && f1[1] == '\0'))
-      {  error("%s:%d: unknown row type `%s' in field 1", fname, seqn,
+      {  print("%s:%d: unknown row type `%s' in field 1", fname, seqn,
             f1);
          return 1;
       }
       if (f2[0] == '\0')
-      {  error("%s:%d: missing row name in field 2", fname, seqn);
+      {  print("%s:%d: missing row name in field 2", fname, seqn);
          return 1;
       }
       if (f3[0] != '\0' || f4[0] != '\0' || f5[0] != '\0'
          || f6[0] != '\0')
-      {  error("%s:%d: invalid data in fields 3-6", fname, seqn);
+      {  print("%s:%d: invalid data in fields 3-6", fname, seqn);
          return 1;
       }
       /* create new row */
       if (find_by_key(t_row, f2))
-      {  error("%s:%d: row `%s' multiply specified", fname, seqn, f2);
+      {  print("%s:%d: row `%s' multiply specified", fname, seqn, f2);
          return 1;
       }
       row = get_atomv(mps->pool, sizeof(MPSROW));
@@ -461,17 +459,17 @@ loop: /* process the next data card */
       /* process optional INTORG/INTEND markers */
       if (strcmp(f3, "'MARKER'") == 0)
       {  if (t_xxx != t_col)
-         {  error("%s:%d): invalid use of marker in RHS or RANGES secti"
+         {  print("%s:%d): invalid use of marker in RHS or RANGES secti"
                "on", fname, seqn);
             return 1;
          }
          if (!(f1[0] == '\0' && f4[0] == '\0' && f6[0] == '\0'))
-         {  error("%s:%d: invalid data in fields 1, 4, or 6", fname,
+         {  print("%s:%d: invalid data in fields 1, 4, or 6", fname,
                seqn);
             return 1;
          }
          if (f2[0] == '\0')
-         {  error("%s:%d: missing marker name in field 2", fname, seqn);
+         {  print("%s:%d: missing marker name in field 2", fname, seqn);
             return 1;
          }
          if (strcmp(f5, "'INTORG'") == 0)
@@ -479,19 +477,19 @@ loop: /* process the next data card */
          else if (strcmp(f5, "'INTEND'") == 0)
             flag = 0;
          else
-         {  error("%s:%d: unknown marker in field 5", fname, seqn);
+         {  print("%s:%d: unknown marker in field 5", fname, seqn);
             return 1;
          }
          goto skip;
       }
       /* process the data card */
       if (f1[0] != '\0')
-      {  error("%s:%d: invalid data in field 1", fname, seqn);
+      {  print("%s:%d: invalid data in field 1", fname, seqn);
          return 1;
       }
       if (f2[0] == '\0') strcpy(f2, name);
       if (f2[0] == '\0')
-      {  error("%s:%d: missing column name in field 2", fname, seqn);
+      {  print("%s:%d: missing column name in field 2", fname, seqn);
          return 1;
       }
       strcpy(name, f2);
@@ -515,7 +513,7 @@ loop: /* process the next data card */
          together as specified by MPS format, although such restriction
          is not essential for this routine */
       if (node->type < t_xxx->size)
-      {  error("%s:%d: %s `%s' multiply specified", fname, seqn,
+      {  print("%s:%d: %s `%s' multiply specified", fname, seqn,
             t_xxx == t_col ? "column" :
             t_xxx == t_rhs ? "right-hand side vector" :
             t_xxx == t_rng ? "range vector" : "???", f2);
@@ -524,21 +522,21 @@ loop: /* process the next data card */
 #endif
       /* process the first row-element pair (fields 3 and 4) */
       if (f3[0] == '\0')
-      {  error("%s:%d: missing row name in field 3", fname, seqn);
+      {  print("%s:%d: missing row name in field 3", fname, seqn);
          return 1;
       }
       if (f4[0] == '\0')
-      {  error("%s:%d: missing value in field 4", fname, seqn);
+      {  print("%s:%d: missing value in field 4", fname, seqn);
          return 1;
       }
       /* create new column or vector element */
       ref = find_by_key(t_row, f3);
       if (ref == NULL)
-      {  error("%s:%d: row `%s' not found", fname, seqn, f3);
+      {  print("%s:%d: row `%s' not found", fname, seqn, f3);
          return 1;
       }
       if (str2dbl(f4, &val))
-      {  error("%s:%d: invalid value `%s'", fname, seqn, f4);
+      {  print("%s:%d: invalid value `%s'", fname, seqn, f4);
          return 1;
       }
       cqe = get_atomv(mps->pool, sizeof(MPSCQE));
@@ -549,21 +547,21 @@ loop: /* process the next data card */
       /* process the second row-element pair (fields 5 and 6) */
       if (f5[0] == '\0' && f6[0] == '\0') goto skip;
       if (f5[0] == '\0')
-      {  error("%s:%d: missing row name in field 5", fname, seqn);
+      {  print("%s:%d: missing row name in field 5", fname, seqn);
          return 1;
       }
       if (f6[0] == '\0')
-      {  error("%s:%d: missing value in filed 6", fname, seqn);
+      {  print("%s:%d: missing value in filed 6", fname, seqn);
          return 1;
       }
       /* create new column or vector element */
       ref = find_by_key(t_row, f5);
       if (ref == NULL)
-      {  error("%s:%d: row `%s' not found", fname, seqn, f5);
+      {  print("%s:%d: row `%s' not found", fname, seqn, f5);
          return 1;
       }
       if (str2dbl(f6, &val))
-      {  error("%s:%d: invalid value `%s'", fname, seqn, f6);
+      {  print("%s:%d: invalid value `%s'", fname, seqn, f6);
          return 1;
       }
       cqe = get_atomv(mps->pool, sizeof(MPSCQE));
@@ -593,35 +591,35 @@ loop: /* process the next data card */
       if (card[0] != ' ') goto done;
       if (split_card()) return 1;
       if (strcmp(f3, "'MARKER'") == 0)
-      {  error("%s:%d: invalid use of marker in BOUNDS section", fname,
+      {  print("%s:%d: invalid use of marker in BOUNDS section", fname,
             seqn);
          return 1;
       }
       if (f1[0] == '\0')
-      {  error("%s:%d: missing bound type in field 1", fname, seqn);
+      {  print("%s:%d: missing bound type in field 1", fname, seqn);
          return 1;
       }
       if (strcmp(f1, "LO") && strcmp(f1, "UP") && strcmp(f1, "FX")
          && strcmp(f1, "FR") && strcmp(f1, "MI") && strcmp(f1, "PL")
          && strcmp(f1, "UI") && strcmp(f1, "BV"))
-      {  error("%s:%d: unknown bound type `%s' in field 1", fname, seqn,
+      {  print("%s:%d: unknown bound type `%s' in field 1", fname, seqn,
             f1);
          return 1;
       }
       if (f2[0] == '\0') strcpy(f2, name);
       strcpy(name, f2);
       if (f3[0] == '\0')
-      {  error("%s:%d: missing column name in field 3", fname, seqn);
+      {  print("%s:%d: missing column name in field 3", fname, seqn);
          return 1;
       }
       if (f4[0] == '\0')
       if (!strcmp(f1, "LO") || !strcmp(f1, "UP") || !strcmp(f1, "FX")
          || !strcmp(f1, "UI"))
-      {  error("%s:%d: missing value in field 4", fname, seqn);
+      {  print("%s:%d: missing value in field 4", fname, seqn);
          return 1;
       }
       if (f5[0] != '\0' && f6[0] != '\0')
-      {  error("%s:%d: invalid data in field 5-6", fname, seqn);
+      {  print("%s:%d: invalid data in field 5-6", fname, seqn);
          return 1;
       }
       /* search for bound vector specified in field 2 */
@@ -642,7 +640,7 @@ loop: /* process the next data card */
          together as specified by MPS format, although such restriction
          is not essential for this routine */
       if (node->type < t_bnd->size)
-      {  error("%s:%d: bound vector `%s' multiply specified", fname,
+      {  print("%s:%d: bound vector `%s' multiply specified", fname,
             seqn, f2);
          return 1;
       }
@@ -650,12 +648,12 @@ loop: /* process the next data card */
       /* process column-element pair */
       ref = find_by_key(t_col, f3);
       if (ref == NULL)
-      {  error("%s:%d: column `%s' not found", fname, seqn, f3);
+      {  print("%s:%d: column `%s' not found", fname, seqn, f3);
          return 1;
       }
       val = 0.0;
       if (f4[0] != '\0' && str2dbl(f4, &val))
-      {  error("%s:%d: invalid value `%s'", fname, seqn, f4);
+      {  print("%s:%d: invalid value `%s'", fname, seqn, f4);
          return 1;
       }
       /* create new bound vector element */
@@ -692,42 +690,42 @@ loop: /* process the next data card */
       if (card[0] != ' ') goto done;
       if (split_card()) return 1;
       if (strcmp(f3, "'MARKER'") == 0)
-      {  error("%s:%d: invalid use of marker on QUADOBJ section",
+      {  print("%s:%d: invalid use of marker on QUADOBJ section",
             fname, seqn);
          return 1;
       }
       if (!(f1[0] == '\0' && f5[0] == '\0' && f6[0] == '\0'))
-      {  error("%s:%d: invalid data in fields 1, 5, or 6",
+      {  print("%s:%d: invalid data in fields 1, 5, or 6",
             fname, seqn);
          return 1;
       }
       if (f2[0] == '\0')
-      {  error("%s:%d: missing first column name in field 2",
+      {  print("%s:%d: missing first column name in field 2",
             fname, seqn);
          return 1;
       }
       if (f3[0] == '\0')
-      {  error("%s:%d: missing second column name in field 3",
+      {  print("%s:%d: missing second column name in field 3",
             fname, seqn);
          return 1;
       }
       if (f4[0] == '\0')
-      {  error("%s:%d: missing value in field 4", fname, seqn);
+      {  print("%s:%d: missing value in field 4", fname, seqn);
          return 1;
       }
       ref1 = find_by_key(t_col, f2);
       if (ref1 == NULL)
-      {  error("%s:%d: column `%s' not found", fname, seqn, f2);
+      {  print("%s:%d: column `%s' not found", fname, seqn, f2);
          return 1;
       }
       ref2 = find_by_key(t_col, f3);
       if (ref2 == NULL)
-      {  error("%s:%d: column `%s' not found", fname, seqn, f3);
+      {  print("%s:%d: column `%s' not found", fname, seqn, f3);
          return 1;
       }
       val = 0.0;
       if (f4[0] != '\0' && str2dbl(f4, &val))
-      {  error("%s:%d: invalid value `%s'", fname, seqn, f4);
+      {  print("%s:%d: invalid value `%s'", fname, seqn, f4);
          return 1;
       }
       /* create new quadratic form element */
@@ -822,7 +820,7 @@ LP *mps_to_lp(MPS *mps, char *obj, char *rhs, char *rng, char *bnd)
          else if (strcmp(row->type, "E") == 0)
             lp->type[k] = 'S';
          else
-         {  error("mps_to_lp: row `%s' has unknown type `%s'",
+         {  print("mps_to_lp: row `%s' has unknown type `%s'",
                row->name, row->type);
             goto fail;
          }
@@ -847,7 +845,7 @@ LP *mps_to_lp(MPS *mps, char *obj, char *rhs, char *rng, char *bnd)
          }
       }
       if (obj[0] != '\0' && objrow == 0)
-      {  error("mps_to_lp: objective function row `%s' not found",
+      {  print("mps_to_lp: objective function row `%s' not found",
             obj);
          goto fail;
       }
@@ -864,7 +862,7 @@ LP *mps_to_lp(MPS *mps, char *obj, char *rhs, char *rng, char *bnd)
       {  for (v = 1; v <= mps->n_rhs; v++)
             if (strcmp(mps->rhs[v]->name, rhs) == 0) break;
          if (v > mps->n_rhs)
-         {  error("mps_to_lp: right-hand side vector `%s' not found",
+         {  print("mps_to_lp: right-hand side vector `%s' not found",
                rhs);
             goto fail;
          }
@@ -904,7 +902,7 @@ LP *mps_to_lp(MPS *mps, char *obj, char *rhs, char *rng, char *bnd)
       {  for (v = 1; v <= mps->n_rng; v++)
             if (strcmp(mps->rng[v]->name, rng) == 0) break;
          if (v > mps->n_rng)
-         {  error("mps_to_lp: range vector `%s' not found", rng);
+         {  print("mps_to_lp: range vector `%s' not found", rng);
             goto fail;
          }
       }
@@ -914,7 +912,7 @@ LP *mps_to_lp(MPS *mps, char *obj, char *rhs, char *rng, char *bnd)
             k = i; /* x[k] = i-th auxiliary variable */
             row = mps->row[i];
             if (strcmp(row->type, "N") == 0)
-            {  error("mps_to_lp: range vector entry refers to row `%s' "
+            {  print("mps_to_lp: range vector entry refers to row `%s' "
                   " of N type", row->name);
                goto fail;
             }
@@ -938,7 +936,7 @@ decr:          lp->lb[k] = lp->ub[k] - fabs(cqe->val);
       {  for (v = 1; v <= mps->n_bnd; v++)
             if (strcmp(mps->bnd[v]->name, bnd) == 0) break;
          if (v > mps->n_bnd)
-         {  error("mps_to_lp: bound vector `%s' not found", bnd);
+         {  print("mps_to_lp: bound vector `%s' not found", bnd);
             goto fail;
          }
       }
@@ -972,7 +970,7 @@ decr:          lp->lb[k] = lp->ub[k] - fabs(cqe->val);
                lp->kind[j] = 1;
             }
             else
-            {  error("mps_to_lp: bound vector entry for column `%s' has"
+            {  print("mps_to_lp: bound vector entry for column `%s' has"
                   " unknown type `%s'", col->name, bqe->type);
                goto fail;
             }
