@@ -26,7 +26,6 @@
 #include <float.h>
 #include <math.h>
 #include <stddef.h>
-#include "glpduff.h"
 #include "glprsm.h"
 
 /*----------------------------------------------------------------------
@@ -529,6 +528,7 @@ err2:                fault("check_rsm: invalid tag of non-basic row/col"
       return;
 }
 
+#if 0 /* 3.0.7 */
 /*----------------------------------------------------------------------
 -- crash_aa() - crash augmented constraint matrix.
 --
@@ -841,6 +841,7 @@ int crash_aa(MAT *A, int tag[], PER *P, PER *Q)
       /* returns to the calling program */
       return ret;
 }
+#endif
 
 /*----------------------------------------------------------------------
 -- create_rsm - create revised simplex method common block.
