@@ -23,6 +23,7 @@
 -- 02111-1307, USA.
 ----------------------------------------------------------------------*/
 
+#include <math.h>
 #include <stddef.h>
 #include "glprsm.h"
 
@@ -619,6 +620,26 @@ int rsm_dual(RSM *rsm, int (*monit)(void), double c[],
          eval_zeta(rsm, p, zeta);
          /* compute p-th (pivot) row of the simplex table */
          eval_row(rsm, zeta, ap);
+#if 0 /* 3.0.9; hmm... either ap is computed quite accurately or the
+         residual can't be detected due to zero bounds of xN */
+         {  int j, k;
+            double sum = 0.0, alpha;
+            for (j = 1; j <= n; j++)
+            {  alpha = ap[j];
+               if (alpha != 0.0)
+               {  k = rsm->indn[j]; /* x[k] = xN[j] */
+                  switch (rsm->tagn[j])
+                  {  case 'L': sum += alpha * rsm->lb[k]; break;
+                     case 'U': sum += alpha * rsm->ub[k]; break;
+                     case 'F': break;
+                     case 'S': sum += alpha * rsm->lb[k]; break;
+                  }
+               }
+            }
+            if (fabs(bbar[p] - sum) / (1.0 + fabs(sum)) > 1e-6)
+               print("bbar = %g; sum = %g", bbar[p], sum);
+         }
+#endif
          /* choose non-basic variable xN[q] */
          if (!relax)
          {  /* use standard ratio test */
