@@ -741,6 +741,7 @@ void spm_set_row(SPM *A, int i, int len, int ndx[], double val[],
       for (t = 0; t < len; t++)
       {  i_ptr = A_ptr[i] + t;
          j = m + A_ndx[i_ptr];
+         aij = A_val[i_ptr];
          /* check if there is an element with the same row index in the
             j-th column (if such element exists, it is the last element
             in the column list) */
@@ -761,7 +762,7 @@ void spm_set_row(SPM *A, int i, int len, int ndx[], double val[],
          /* add new a[i,j] to the end of the j-th column list */
          j_ptr = A_ptr[j] + (A_len[j]++);
          A_ndx[j_ptr] = i;
-         A_val[j_ptr] = A_val[i_ptr];
+         A_val[j_ptr] = aij;
       }
       return;
 }
@@ -861,6 +862,7 @@ void spm_set_col(SPM *A, int j, int len, int ndx[], double val[],
       for (t = 0; t < len; t++)
       {  j_ptr = A_ptr[m+j] + t;
          i = A_ndx[j_ptr];
+         aij = A_val[j_ptr];
          /* check if there is an element with the same column index in
             the i-th row (if such element exists, it is the last element
             in the row list) */
@@ -881,7 +883,7 @@ void spm_set_col(SPM *A, int j, int len, int ndx[], double val[],
          /* add new a[i,j] to the end of the i-th row list */
          i_ptr = A_ptr[i] + (A_len[i]++);
          A_ndx[i_ptr] = j;
-         A_val[i_ptr] = A_val[j_ptr];
+         A_val[i_ptr] = aij;
       }
       return;
 }
